@@ -11,7 +11,9 @@ import { TestRig, printDebugInfo, validateModelOutput } from './test-helper.js';
 describe(WEB_SEARCH_TOOL_NAME, () => {
   it('should be able to search the web', async () => {
     const rig = new TestRig();
-    await rig.setup('should be able to search the web');
+    await rig.setup('should be able to search the web', {
+      settings: { tools: { core: [WEB_SEARCH_TOOL_NAME] } },
+    });
 
     let result;
     try {
