@@ -346,6 +346,17 @@ describe('SettingsSchema', () => {
       ).toBe('Enable preview features (e.g., preview models).');
     });
 
+    it('should have enableAgents setting in schema', () => {
+      const setting = getSettingsSchema().experimental.properties.enableAgents;
+      expect(setting).toBeDefined();
+      expect(setting.type).toBe('boolean');
+      expect(setting.category).toBe('Experimental');
+      expect(setting.default).toBe(false);
+      expect(setting.requiresRestart).toBe(true);
+      expect(setting.showInDialog).toBe(false);
+      expect(setting.description).toBe('Enable local and remote subagents.');
+    });
+
     it('should have isModelAvailabilityServiceEnabled setting in schema', () => {
       const setting =
         getSettingsSchema().experimental.properties
