@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
+import * as glob from 'glob';
 import * as path from 'node:path';
 import type { Config } from '@google/gemini-cli-core';
 import { GEMINI_DIR, Storage } from '@google/gemini-cli-core';
@@ -70,11 +71,18 @@ vi.mock('@google/gemini-cli-core', async (importOriginal) => {
   };
 });
 
+vi.mock('glob', () => ({
+  glob: vi.fn(),
+}));
+
 describe('FileCommandLoader', () => {
   const signal: AbortSignal = new AbortController().signal;
 
-  beforeEach(() => {
+  beforeEach(async () => {
     vi.clearAllMocks();
+    const { glob: actualGlob } =
+      await vi.importActual<typeof import('glob')>('glob');
+    vi.mocked(glob.glob).mockImplementation(actualGlob);
     mockShellProcess.mockImplementation(
       (prompt: PromptPipelineContent, context: CommandContext) => {
         const userArgsRaw = context?.invocation?.args || '';
@@ -842,6 +850,52 @@ describe('FileCommandLoader', () => {
         assert.fail('Incorrect action type');
       }
     });
+
+    it('correctly loads extensionId for extension commands', async () => {
+      const extensionId = 'my-test-ext-id-123';
+      const extensionDir = path.join(
+        process.cwd(),
+        GEMINI_DIR,
+        'extensions',
+        'my-test-ext',
+      );
+
+      mock({
+        [extensionDir]: {
+          'gemini-extension.json': JSON.stringify({
+            name: 'my-test-ext',
+            id: extensionId,
+            version: '1.0.0',
+          }),
+          commands: {
+            'my-cmd.toml': 'prompt = "My test command"',
+          },
+        },
+      });
+
+      const mockConfig = {
+        getProjectRoot: vi.fn(() => process.cwd()),
+        getExtensions: vi.fn(() => [
+          {
+            name: 'my-test-ext',
+            id: extensionId,
+            version: '1.0.0',
+            isActive: true,
+            path: extensionDir,
+          },
+        ]),
+        getFolderTrust: vi.fn(() => false),
+        isTrustedFolder: vi.fn(() => false),
+      } as unknown as Config;
+      const loader = new FileCommandLoader(mockConfig);
+      const commands = await loader.loadCommands(signal);
+
+      expect(commands).toHaveLength(1);
+      const command = commands[0];
+      expect(command.name).toBe('my-cmd');
+      expect(command.extensionName).toBe('my-test-ext');
+      expect(command.extensionId).toBe(extensionId);
+    });
   });
 
   describe('Argument Handling Integration (via ShellProcessor)', () => {
@@ -1242,4 +1296,45 @@ describe('FileCommandLoader', () => {
       expect(commands).toHaveLength(0);
     });
   });
+
+  describe('Aborted signal', () => {
+    it('does not log errors if the signal is aborted', async () => {
+      const controller = new AbortController();
+      const abortSignal = controller.signal;
+
+      const consoleErrorSpy = vi
+        .spyOn(console, 'error')
+        .mockImplementation(() => {});
+
+      const mockConfig = {
+        getProjectRoot: vi.fn(() => '/path/to/project'),
+        getExtensions: vi.fn(() => []),
+        getFolderTrust: vi.fn(() => false),
+        isTrustedFolder: vi.fn(() => false),
+      } as unknown as Config;
+
+      // Set up mock-fs so that the loader attempts to read a directory.
+      const userCommandsDir = Storage.getUserCommandsDir();
+      mock({
+        [userCommandsDir]: {
+          'test1.toml': 'prompt = "Prompt 1"',
+        },
+      });
+
+      const loader = new FileCommandLoader(mockConfig);
+
+      // Mock glob to throw an AbortError
+      const abortError = new DOMException('Aborted', 'AbortError');
+      vi.mocked(glob.glob).mockImplementation(async () => {
+        controller.abort(); // Ensure the signal is aborted when the service checks
+        throw abortError;
+      });
+
+      await loader.loadCommands(abortSignal);
+
+      expect(consoleErrorSpy).not.toHaveBeenCalled();
+
+      consoleErrorSpy.mockRestore();
+    });
+  });
 });
