@@ -8,7 +8,7 @@ import type { MockInstance } from 'vitest';
 import { vi, describe, it, expect, beforeEach, afterEach } from 'vitest';
 import { ideCommand } from './ideCommand.js';
 import { type CommandContext } from './types.js';
-import { DetectedIde } from '@google/gemini-cli-core';
+import { IDE_DEFINITIONS } from '@google/gemini-cli-core';
 import * as core from '@google/gemini-cli-core';
 
 vi.mock('@google/gemini-cli-core', async (importOriginal) => {
@@ -36,11 +36,14 @@ describe('ideCommand', () => {
       disconnect: vi.fn(),
       connect: vi.fn(),
       getCurrentIde: vi.fn(),
-      getDetectedIdeDisplayName: vi.fn(),
       getConnectionStatus: vi.fn(),
+      getDetectedIdeDisplayName: vi.fn(),
     } as unknown as core.IdeClient;
 
     vi.mocked(core.IdeClient.getInstance).mockResolvedValue(mockIdeClient);
+    vi.mocked(mockIdeClient.getDetectedIdeDisplayName).mockReturnValue(
+      'VS Code',
+    );
 
     mockContext = {
       ui: {
@@ -66,9 +69,8 @@ describe('ideCommand', () => {
   });
 
   it('should return the ide command', async () => {
-    vi.mocked(mockIdeClient.getCurrentIde).mockReturnValue(DetectedIde.VSCode);
-    vi.mocked(mockIdeClient.getDetectedIdeDisplayName).mockReturnValue(
-      'VS Code',
+    vi.mocked(mockIdeClient.getCurrentIde).mockReturnValue(
+      IDE_DEFINITIONS.vscode,
     );
     vi.mocked(mockIdeClient.getConnectionStatus).mockReturnValue({
       status: core.IDEConnectionStatus.Disconnected,
@@ -83,9 +85,8 @@ describe('ideCommand', () => {
   });
 
   it('should show disable command when connected', async () => {
-    vi.mocked(mockIdeClient.getCurrentIde).mockReturnValue(DetectedIde.VSCode);
-    vi.mocked(mockIdeClient.getDetectedIdeDisplayName).mockReturnValue(
-      'VS Code',
+    vi.mocked(mockIdeClient.getCurrentIde).mockReturnValue(
+      IDE_DEFINITIONS.vscode,
     );
     vi.mocked(mockIdeClient.getConnectionStatus).mockReturnValue({
       status: core.IDEConnectionStatus.Connected,
@@ -100,10 +101,7 @@ describe('ideCommand', () => {
   describe('status subcommand', () => {
     beforeEach(() => {
       vi.mocked(mockIdeClient.getCurrentIde).mockReturnValue(
-        DetectedIde.VSCode,
-      );
-      vi.mocked(mockIdeClient.getDetectedIdeDisplayName).mockReturnValue(
-        'VS Code',
+        IDE_DEFINITIONS.vscode,
       );
     });
 
@@ -177,10 +175,7 @@ describe('ideCommand', () => {
     const mockInstall = vi.fn();
     beforeEach(() => {
       vi.mocked(mockIdeClient.getCurrentIde).mockReturnValue(
-        DetectedIde.VSCode,
-      );
-      vi.mocked(mockIdeClient.getDetectedIdeDisplayName).mockReturnValue(
-        'VS Code',
+        IDE_DEFINITIONS.vscode,
       );
       vi.mocked(mockIdeClient.getConnectionStatus).mockReturnValue({
         status: core.IDEConnectionStatus.Disconnected,
@@ -211,7 +206,7 @@ describe('ideCommand', () => {
       await vi.runAllTimersAsync();
       await actionPromise;
 
-      expect(core.getIdeInstaller).toHaveBeenCalledWith('vscode');
+      expect(core.getIdeInstaller).toHaveBeenCalledWith(IDE_DEFINITIONS.vscode);
       expect(mockInstall).toHaveBeenCalled();
       expect(mockContext.ui.addItem).toHaveBeenCalledWith(
         expect.objectContaining({
@@ -249,7 +244,7 @@ describe('ideCommand', () => {
         '',
       );
 
-      expect(core.getIdeInstaller).toHaveBeenCalledWith('vscode');
+      expect(core.getIdeInstaller).toHaveBeenCalledWith(IDE_DEFINITIONS.vscode);
       expect(mockInstall).toHaveBeenCalled();
       expect(mockContext.ui.addItem).toHaveBeenCalledWith(
         expect.objectContaining({
