@@ -5,12 +5,19 @@
  */
 
 import type React from 'react';
-import { Box, Text } from 'ink';
-import Gradient from 'ink-gradient';
-import { theme } from '../semantic-colors.js';
-import { shortAsciiLogo, longAsciiLogo, tinyAsciiLogo } from './AsciiArt.js';
+import { Box } from 'ink';
+import { ThemedGradient } from './ThemedGradient.js';
+import {
+  shortAsciiLogo,
+  longAsciiLogo,
+  tinyAsciiLogo,
+  shortAsciiLogoIde,
+  longAsciiLogoIde,
+  tinyAsciiLogoIde,
+} from './AsciiArt.js';
 import { getAsciiArtWidth } from '../utils/textUtils.js';
 import { useTerminalSize } from '../hooks/useTerminalSize.js';
+import { getTerminalProgram } from '../utils/terminalSetup.js';
 
 interface HeaderProps {
   customAsciiArt?: string; // For user-defined ASCII art
@@ -24,6 +31,7 @@ export const Header: React.FC<HeaderProps> = ({
   nightly,
 }) => {
   const { columns: terminalWidth } = useTerminalSize();
+  const isIde = getTerminalProgram();
   let displayTitle;
   const widthOfLongLogo = getAsciiArtWidth(longAsciiLogo);
   const widthOfShortLogo = getAsciiArtWidth(shortAsciiLogo);
@@ -31,11 +39,11 @@ export const Header: React.FC<HeaderProps> = ({
   if (customAsciiArt) {
     displayTitle = customAsciiArt;
   } else if (terminalWidth >= widthOfLongLogo) {
-    displayTitle = longAsciiLogo;
+    displayTitle = isIde ? longAsciiLogoIde : longAsciiLogo;
   } else if (terminalWidth >= widthOfShortLogo) {
-    displayTitle = shortAsciiLogo;
+    displayTitle = isIde ? shortAsciiLogoIde : shortAsciiLogo;
   } else {
-    displayTitle = tinyAsciiLogo;
+    displayTitle = isIde ? tinyAsciiLogoIde : tinyAsciiLogo;
   }
 
   const artWidth = getAsciiArtWidth(displayTitle);
@@ -47,22 +55,10 @@ export const Header: React.FC<HeaderProps> = ({
       flexShrink={0}
       flexDirection="column"
     >
-      {theme.ui.gradient ? (
-        <Gradient colors={theme.ui.gradient}>
-          <Text>{displayTitle}</Text>
-        </Gradient>
-      ) : (
-        <Text>{displayTitle}</Text>
-      )}
+      <ThemedGradient>{displayTitle}</ThemedGradient>
       {nightly && (
         <Box width="100%" flexDirection="row" justifyContent="flex-end">
-          {theme.ui.gradient ? (
-            <Gradient colors={theme.ui.gradient}>
-              <Text>v{version}</Text>
-            </Gradient>
-          ) : (
-            <Text>v{version}</Text>
-          )}
+          <ThemedGradient>v{version}</ThemedGradient>
         </Box>
       )}
     </Box>
