@@ -5,24 +5,28 @@
  */
 
 import { describe, it, expect } from 'vitest';
-import { render } from 'ink-testing-library';
+import { render } from '../../test-utils/render.js';
 import { QueuedMessageDisplay } from './QueuedMessageDisplay.js';
 
 describe('QueuedMessageDisplay', () => {
   it('renders nothing when message queue is empty', () => {
-    const { lastFrame } = render(<QueuedMessageDisplay messageQueue={[]} />);
+    const { lastFrame, unmount } = render(
+      <QueuedMessageDisplay messageQueue={[]} />,
+    );
 
     expect(lastFrame()).toBe('');
+    unmount();
   });
 
   it('displays single queued message', () => {
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <QueuedMessageDisplay messageQueue={['First message']} />,
     );
 
     const output = lastFrame();
     expect(output).toContain('Queued (press ↑ to edit):');
     expect(output).toContain('First message');
+    unmount();
   });
 
   it('displays multiple queued messages', () => {
@@ -32,7 +36,7 @@ describe('QueuedMessageDisplay', () => {
       'Third queued message',
     ];
 
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <QueuedMessageDisplay messageQueue={messageQueue} />,
     );
 
@@ -41,6 +45,7 @@ describe('QueuedMessageDisplay', () => {
     expect(output).toContain('First queued message');
     expect(output).toContain('Second queued message');
     expect(output).toContain('Third queued message');
+    unmount();
   });
 
   it('shows overflow indicator when more than 3 messages are queued', () => {
@@ -52,7 +57,7 @@ describe('QueuedMessageDisplay', () => {
       'Message 5',
     ];
 
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <QueuedMessageDisplay messageQueue={messageQueue} />,
     );
 
@@ -64,17 +69,19 @@ describe('QueuedMessageDisplay', () => {
     expect(output).toContain('... (+2 more)');
     expect(output).not.toContain('Message 4');
     expect(output).not.toContain('Message 5');
+    unmount();
   });
 
   it('normalizes whitespace in messages', () => {
     const messageQueue = ['Message   with\tmultiple\n  whitespace'];
 
-    const { lastFrame } = render(
+    const { lastFrame, unmount } = render(
       <QueuedMessageDisplay messageQueue={messageQueue} />,
     );
 
     const output = lastFrame();
     expect(output).toContain('Queued (press ↑ to edit):');
     expect(output).toContain('Message with multiple whitespace');
+    unmount();
   });
 });
