@@ -10,6 +10,7 @@ import { MarkdownDisplay } from '../../utils/MarkdownDisplay.js';
 import { theme } from '../../semantic-colors.js';
 import { SCREEN_READER_MODEL_PREFIX } from '../../textConstants.js';
 import { useUIState } from '../../contexts/UIStateContext.js';
+import { useAlternateBuffer } from '../../hooks/useAlternateBuffer.js';
 
 interface GeminiMessageProps {
   text: string;
@@ -28,6 +29,7 @@ export const GeminiMessage: React.FC<GeminiMessageProps> = ({
   const prefix = '✦ ';
   const prefixWidth = prefix.length;
 
+  const isAlternateBuffer = useAlternateBuffer();
   return (
     <Box flexDirection="row">
       <Box width={prefixWidth}>
@@ -39,7 +41,9 @@ export const GeminiMessage: React.FC<GeminiMessageProps> = ({
         <MarkdownDisplay
           text={text}
           isPending={isPending}
-          availableTerminalHeight={availableTerminalHeight}
+          availableTerminalHeight={
+            isAlternateBuffer ? undefined : availableTerminalHeight
+          }
           terminalWidth={terminalWidth}
           renderMarkdown={renderMarkdown}
         />
