@@ -5,9 +5,10 @@
  */
 
 import { useEffect } from 'react';
-
-const ENABLE_BRACKETED_PASTE = '\x1b[?2004h';
-const DISABLE_BRACKETED_PASTE = '\x1b[?2004l';
+import {
+  disableBracketedPaste,
+  enableBracketedPaste,
+} from '../utils/bracketedPaste.js';
 
 /**
  * Enables and disables bracketed paste mode in the terminal.
@@ -17,11 +18,11 @@ const DISABLE_BRACKETED_PASTE = '\x1b[?2004l';
  */
 export const useBracketedPaste = () => {
   const cleanup = () => {
-    process.stdout.write(DISABLE_BRACKETED_PASTE);
+    disableBracketedPaste();
   };
 
   useEffect(() => {
-    process.stdout.write(ENABLE_BRACKETED_PASTE);
+    enableBracketedPaste();
 
     process.on('exit', cleanup);
     process.on('SIGINT', cleanup);
