@@ -5,13 +5,9 @@
  */
 
 import { useState } from 'react';
-import {
-  isKittyProtocolEnabled,
-  isKittyProtocolSupported,
-} from '../utils/kittyProtocolDetector.js';
+import { isKittyProtocolEnabled } from '../utils/kittyProtocolDetector.js';
 
 export interface KittyProtocolStatus {
-  supported: boolean;
   enabled: boolean;
   checking: boolean;
 }
@@ -22,7 +18,6 @@ export interface KittyProtocolStatus {
  */
 export function useKittyKeyboardProtocol(): KittyProtocolStatus {
   const [status] = useState<KittyProtocolStatus>({
-    supported: isKittyProtocolSupported(),
     enabled: isKittyProtocolEnabled(),
     checking: false,
   });
