@@ -4,14 +4,13 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-/** @vitest-environment jsdom */
-
 import { describe, it, expect, vi } from 'vitest';
-import { renderHook, waitFor } from '@testing-library/react';
+import { act, useState } from 'react';
+import { renderHook } from '../../test-utils/render.js';
+import { waitFor } from '../../test-utils/async.js';
 import { useSlashCompletion } from './useSlashCompletion.js';
 import type { CommandContext, SlashCommand } from '../commands/types.js';
 import { CommandKind } from '../commands/types.js';
-import { useState } from 'react';
 import type { Suggestion } from '../components/SuggestionsDisplay.js';
 
 // Test utility type and helper function for creating test SlashCommands
@@ -184,7 +183,8 @@ describe('useSlashCompletion', () => {
         createTestCommand({
           name: 'stats',
           altNames: ['usage'],
-          description: 'check session stats. Usage: /stats [model|tools]',
+          description:
+            'check session stats. Usage: /stats [session|model|tools]',
         }),
         createTestCommand({ name: 'clear', description: 'Clear the screen' }),
         createTestCommand({
@@ -196,44 +196,86 @@ describe('useSlashCompletion', () => {
         }),
         createTestCommand({ name: 'chat', description: 'Manage chat history' }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
+      let result: {
+        current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useTestHarnessForSlashCompletion(
+            true,
+            '/',
+            slashCommands,
+            mockCommandContext,
+          ),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
+      });
 
-      expect(result.current.suggestions.length).toBe(slashCommands.length);
-      expect(result.current.suggestions.map((s) => s.label)).toEqual(
-        expect.arrayContaining(['help', 'clear', 'memory', 'chat', 'stats']),
-      );
+      await act(async () => {
+        await waitFor(() => {
+          expect(result.current.suggestions.length).toBe(slashCommands.length);
+          expect(result.current.suggestions.map((s) => s.label)).toEqual(
+            expect.arrayContaining([
+              'help',
+              'clear',
+              'memory',
+              'chat',
+              'stats',
+            ]),
+          );
+        });
+      });
+      unmount!();
     });
 
     it('should filter commands based on partial input', async () => {
       const slashCommands = [
         createTestCommand({ name: 'memory', description: 'Manage memory' }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/mem',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
+      const setSuggestions = vi.fn();
+      const setIsLoadingSuggestions = vi.fn();
+      const setIsPerfectMatch = vi.fn();
 
-      await waitFor(() => {
-        expect(result.current.suggestions).toEqual([
-          {
-            label: 'memory',
-            value: 'memory',
-            description: 'Manage memory',
-            commandKind: CommandKind.BUILT_IN,
-          },
-        ]);
+      let result: {
+        current: { completionStart: number; completionEnd: number };
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useSlashCompletion({
+            enabled: true,
+            query: '/mem',
+            slashCommands,
+            commandContext: mockCommandContext,
+            setSuggestions,
+            setIsLoadingSuggestions,
+            setIsPerfectMatch,
+          }),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
       });
+
+      await act(async () => {
+        await waitFor(() => {
+          expect(setSuggestions).toHaveBeenCalledWith([
+            {
+              label: 'memory',
+              value: 'memory',
+              description: 'Manage memory',
+              commandKind: CommandKind.BUILT_IN,
+            },
+          ]);
+          expect(result.current.completionStart).toBe(1);
+          expect(result.current.completionEnd).toBe(4);
+        });
+      });
+      await act(async () => {
+        await new Promise((resolve) => setTimeout(resolve, 50));
+      });
+      unmount!();
     });
 
     it('should suggest commands based on partial altNames', async () => {
@@ -241,28 +283,40 @@ describe('useSlashCompletion', () => {
         createTestCommand({
           name: 'stats',
           altNames: ['usage'],
-          description: 'check session stats. Usage: /stats [model|tools]',
+          description:
+            'check session stats. Usage: /stats [session|model|tools]',
         }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/usag',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
+      let result: {
+        current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useTestHarnessForSlashCompletion(
+            true,
+            '/usage',
+            slashCommands,
+            mockCommandContext,
+          ),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
+      });
 
       await waitFor(() => {
         expect(result.current.suggestions).toEqual([
           {
             label: 'stats',
             value: 'stats',
-            description: 'check session stats. Usage: /stats [model|tools]',
+            description:
+              'check session stats. Usage: /stats [session|model|tools]',
             commandKind: CommandKind.BUILT_IN,
           },
         ]);
+        expect(result.current.completionStart).toBe(1);
       });
+      unmount!();
     });
 
     it('should NOT provide suggestions for a perfectly typed command that is a leaf node', async () => {
@@ -273,16 +327,27 @@ describe('useSlashCompletion', () => {
           action: vi.fn(),
         }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/clear',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
-
-      expect(result.current.suggestions).toHaveLength(0);
+      let result: {
+        current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useTestHarnessForSlashCompletion(
+            true,
+            '/clear',
+            slashCommands,
+            mockCommandContext,
+          ),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
+      });
+      await waitFor(() => {
+        expect(result.current.suggestions).toHaveLength(0);
+        expect(result.current.completionStart).toBe(1);
+      });
+      unmount!();
     });
 
     it.each([['/?'], ['/usage']])(
@@ -298,21 +363,34 @@ describe('useSlashCompletion', () => {
           createTestCommand({
             name: 'stats',
             altNames: ['usage'],
-            description: 'check session stats. Usage: /stats [model|tools]',
+            description:
+              'check session stats. Usage: /stats [session|model|tools]',
             action: vi.fn(),
           }),
         ];
 
-        const { result } = renderHook(() =>
-          useTestHarnessForSlashCompletion(
-            true,
-            query,
-            mockSlashCommands,
-            mockCommandContext,
-          ),
-        );
+        let result: {
+          current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+        };
+        let unmount: () => void;
+        await act(async () => {
+          const hook = renderHook(() =>
+            useTestHarnessForSlashCompletion(
+              true,
+              query,
+              mockSlashCommands,
+              mockCommandContext,
+            ),
+          );
+          result = hook.result;
+          unmount = hook.unmount;
+        });
 
-        expect(result.current.suggestions).toHaveLength(0);
+        await waitFor(() => {
+          expect(result.current.suggestions).toHaveLength(0);
+          expect(result.current.completionStart).toBe(1);
+        });
+        unmount!();
       },
     );
 
@@ -320,32 +398,55 @@ describe('useSlashCompletion', () => {
       const slashCommands = [
         createTestCommand({ name: 'clear', description: 'Clear the screen' }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/clear ',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
+      let result: {
+        current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useTestHarnessForSlashCompletion(
+            true,
+            '/clear ',
+            slashCommands,
+            mockCommandContext,
+          ),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
+      });
 
-      expect(result.current.suggestions).toHaveLength(0);
+      await waitFor(() => {
+        expect(result.current.suggestions).toHaveLength(0);
+      });
+      unmount!();
     });
 
     it('should not provide suggestions for an unknown command', async () => {
       const slashCommands = [
         createTestCommand({ name: 'help', description: 'Show help' }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/unknown-command',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
+      let result: {
+        current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useTestHarnessForSlashCompletion(
+            true,
+            '/unknown-command',
+            slashCommands,
+            mockCommandContext,
+          ),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
+      });
 
-      expect(result.current.suggestions).toHaveLength(0);
+      await waitFor(() => {
+        expect(result.current.suggestions).toHaveLength(0);
+        expect(result.current.completionStart).toBe(1);
+      });
+      unmount!();
     });
 
     it('should not suggest hidden commands', async () => {
@@ -360,17 +461,28 @@ describe('useSlashCompletion', () => {
           hidden: true,
         }),
       ];
-      const { result } = renderHook(() =>
-        useTestHarnessForSlashCompletion(
-          true,
-          '/',
-          slashCommands,
-          mockCommandContext,
-        ),
-      );
+      let result: {
+        current: ReturnType<typeof useTestHarnessForSlashCompletion>;
+      };
+      let unmount: () => void;
+      await act(async () => {
+        const hook = renderHook(() =>
+          useTestHarnessForSlashCompletion(
+            true,
+            '/',
+            slashCommands,
+            mockCommandContext,
+          ),
+        );
+        result = hook.result;
+        unmount = hook.unmount;
+      });
 
-      expect(result.current.suggestions.length).toBe(1);
-      expect(result.current.suggestions[0].label).toBe('visible');
+      await waitFor(() => {
+        expect(result.current.suggestions.length).toBe(1);
+        expect(result.current.suggestions[0].label).toBe('visible');
+      });
+      unmount!();
     });
   });
 
@@ -387,32 +499,35 @@ describe('useSlashCompletion', () => {
         }),
       ];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
-          '/memory',
+          '/memory ',
           slashCommands,
           mockCommandContext,
         ),
       );
 
-      expect(result.current.suggestions).toHaveLength(2);
-      expect(result.current.suggestions).toEqual(
-        expect.arrayContaining([
-          {
-            label: 'show',
-            value: 'show',
-            description: 'Show memory',
-            commandKind: CommandKind.BUILT_IN,
-          },
-          {
-            label: 'add',
-            value: 'add',
-            description: 'Add to memory',
-            commandKind: CommandKind.BUILT_IN,
-          },
-        ]),
-      );
+      await waitFor(() => {
+        expect(result.current.suggestions).toHaveLength(2);
+        expect(result.current.suggestions).toEqual(
+          expect.arrayContaining([
+            {
+              label: 'show',
+              value: 'show',
+              description: 'Show memory',
+              commandKind: CommandKind.BUILT_IN,
+            },
+            {
+              label: 'add',
+              value: 'add',
+              description: 'Add to memory',
+              commandKind: CommandKind.BUILT_IN,
+            },
+          ]),
+        );
+      });
+      unmount();
     });
 
     it('should suggest all sub-commands when the query ends with the parent command and a space', async () => {
@@ -426,7 +541,7 @@ describe('useSlashCompletion', () => {
           ],
         }),
       ];
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/memory ',
@@ -435,23 +550,26 @@ describe('useSlashCompletion', () => {
         ),
       );
 
-      expect(result.current.suggestions).toHaveLength(2);
-      expect(result.current.suggestions).toEqual(
-        expect.arrayContaining([
-          {
-            label: 'show',
-            value: 'show',
-            description: 'Show memory',
-            commandKind: CommandKind.BUILT_IN,
-          },
-          {
-            label: 'add',
-            value: 'add',
-            description: 'Add to memory',
-            commandKind: CommandKind.BUILT_IN,
-          },
-        ]),
-      );
+      await waitFor(() => {
+        expect(result.current.suggestions).toHaveLength(2);
+        expect(result.current.suggestions).toEqual(
+          expect.arrayContaining([
+            {
+              label: 'show',
+              value: 'show',
+              description: 'Show memory',
+              commandKind: CommandKind.BUILT_IN,
+            },
+            {
+              label: 'add',
+              value: 'add',
+              description: 'Add to memory',
+              commandKind: CommandKind.BUILT_IN,
+            },
+          ]),
+        );
+      });
+      unmount();
     });
 
     it('should filter sub-commands by prefix', async () => {
@@ -465,7 +583,7 @@ describe('useSlashCompletion', () => {
           ],
         }),
       ];
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/memory a',
@@ -483,7 +601,9 @@ describe('useSlashCompletion', () => {
             commandKind: CommandKind.BUILT_IN,
           },
         ]);
+        expect(result.current.completionStart).toBe(8);
       });
+      unmount();
     });
 
     it('should provide no suggestions for an invalid sub-command', async () => {
@@ -497,7 +617,7 @@ describe('useSlashCompletion', () => {
           ],
         }),
       ];
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/memory dothisnow',
@@ -505,8 +625,13 @@ describe('useSlashCompletion', () => {
           mockCommandContext,
         ),
       );
-
-      expect(result.current.suggestions).toHaveLength(0);
+      await act(async () => {
+        await waitFor(() => {
+          expect(result.current.suggestions).toHaveLength(0);
+          expect(result.current.completionStart).toBe(8);
+        });
+      });
+      unmount();
     });
   });
 
@@ -538,7 +663,7 @@ describe('useSlashCompletion', () => {
         }),
       ];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/chat resume my-ch',
@@ -547,25 +672,32 @@ describe('useSlashCompletion', () => {
         ),
       );
 
-      await waitFor(() => {
-        expect(mockCompletionFn).toHaveBeenCalledWith(
-          expect.objectContaining({
-            invocation: {
-              raw: '/chat resume my-ch',
-              name: 'resume',
-              args: 'my-ch',
-            },
-          }),
-          'my-ch',
-        );
+      await act(async () => {
+        await waitFor(() => {
+          expect(mockCompletionFn).toHaveBeenCalledWith(
+            expect.objectContaining({
+              invocation: {
+                raw: '/chat resume my-ch',
+                name: 'resume',
+                args: 'my-ch',
+              },
+            }),
+            'my-ch',
+          );
+        });
       });
 
-      await waitFor(() => {
-        expect(result.current.suggestions).toEqual([
-          { label: 'my-chat-tag-1', value: 'my-chat-tag-1' },
-          { label: 'my-chat-tag-2', value: 'my-chat-tag-2' },
-        ]);
+      await act(async () => {
+        await waitFor(() => {
+          expect(result.current.suggestions).toEqual([
+            { label: 'my-chat-tag-1', value: 'my-chat-tag-1' },
+            { label: 'my-chat-tag-2', value: 'my-chat-tag-2' },
+          ]);
+          expect(result.current.completionStart).toBe(13);
+          expect(result.current.isLoadingSuggestions).toBe(false);
+        });
       });
+      unmount();
     });
 
     it('should call command.completion with an empty string when args start with a space', async () => {
@@ -587,7 +719,7 @@ describe('useSlashCompletion', () => {
         }),
       ];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/chat resume ',
@@ -596,52 +728,55 @@ describe('useSlashCompletion', () => {
         ),
       );
 
-      await waitFor(() => {
-        expect(mockCompletionFn).toHaveBeenCalledWith(
-          expect.objectContaining({
-            invocation: {
-              raw: '/chat resume',
-              name: 'resume',
-              args: '',
-            },
-          }),
-          '',
-        );
+      await act(async () => {
+        await waitFor(() => {
+          expect(mockCompletionFn).toHaveBeenCalledWith(
+            expect.objectContaining({
+              invocation: {
+                raw: '/chat resume ',
+                name: 'resume',
+                args: '',
+              },
+            }),
+            '',
+          );
+        });
       });
 
-      await waitFor(() => {
-        expect(result.current.suggestions).toHaveLength(3);
+      await act(async () => {
+        await waitFor(() => {
+          expect(result.current.suggestions).toHaveLength(3);
+          expect(result.current.completionStart).toBe(13);
+        });
       });
+      unmount();
     });
 
     it('should handle completion function that returns null', async () => {
-      const completionFn = vi.fn().mockResolvedValue(null);
+      const mockCompletionFn = vi.fn().mockResolvedValue(null);
+
       const slashCommands = [
         createTestCommand({
-          name: 'chat',
-          description: 'Manage chat history',
-          subCommands: [
-            createTestCommand({
-              name: 'resume',
-              description: 'Resume a saved chat',
-              completion: completionFn,
-            }),
-          ],
+          name: 'test',
+          description: 'Test command',
+          completion: mockCompletionFn,
         }),
       ];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
-          '/chat resume ',
+          '/test arg',
           slashCommands,
           mockCommandContext,
         ),
       );
 
       await waitFor(() => {
-        expect(result.current.suggestions).toHaveLength(0);
+        expect(result.current.suggestions).toEqual([]);
+        expect(result.current.isLoadingSuggestions).toBe(false);
       });
+      unmount();
     });
   });
 
@@ -662,7 +797,7 @@ describe('useSlashCompletion', () => {
         },
       ] as SlashCommand[];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/',
@@ -671,22 +806,25 @@ describe('useSlashCompletion', () => {
         ),
       );
 
-      expect(result.current.suggestions).toEqual(
-        expect.arrayContaining([
-          {
-            label: 'summarize',
-            value: 'summarize',
-            description: 'Summarize content',
-            commandKind: CommandKind.MCP_PROMPT,
-          },
-          {
-            label: 'help',
-            value: 'help',
-            description: 'Show help',
-            commandKind: CommandKind.BUILT_IN,
-          },
-        ]),
-      );
+      await waitFor(() => {
+        expect(result.current.suggestions).toEqual(
+          expect.arrayContaining([
+            {
+              label: 'summarize',
+              value: 'summarize',
+              description: 'Summarize content',
+              commandKind: CommandKind.MCP_PROMPT,
+            },
+            {
+              label: 'help',
+              value: 'help',
+              description: 'Show help',
+              commandKind: CommandKind.BUILT_IN,
+            },
+          ]),
+        );
+      });
+      unmount();
     });
 
     it('should include commandKind when filtering MCP commands by prefix', async () => {
@@ -705,7 +843,7 @@ describe('useSlashCompletion', () => {
         },
       ] as SlashCommand[];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/summ',
@@ -723,7 +861,9 @@ describe('useSlashCompletion', () => {
             commandKind: CommandKind.MCP_PROMPT,
           },
         ]);
+        expect(result.current.completionStart).toBe(1);
       });
+      unmount();
     });
 
     it('should include commandKind for sub-commands', async () => {
@@ -749,7 +889,7 @@ describe('useSlashCompletion', () => {
         },
       ] as SlashCommand[];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/memory',
@@ -758,22 +898,25 @@ describe('useSlashCompletion', () => {
         ),
       );
 
-      expect(result.current.suggestions).toEqual(
-        expect.arrayContaining([
-          {
-            label: 'show',
-            value: 'show',
-            description: 'Show memory',
-            commandKind: CommandKind.BUILT_IN,
-          },
-          {
-            label: 'add',
-            value: 'add',
-            description: 'Add to memory',
-            commandKind: CommandKind.MCP_PROMPT,
-          },
-        ]),
-      );
+      await waitFor(() => {
+        expect(result.current.suggestions).toEqual(
+          expect.arrayContaining([
+            {
+              label: 'show',
+              value: 'show',
+              description: 'Show memory',
+              commandKind: CommandKind.BUILT_IN,
+            },
+            {
+              label: 'add',
+              value: 'add',
+              description: 'Add to memory',
+              commandKind: CommandKind.MCP_PROMPT,
+            },
+          ]),
+        );
+      });
+      unmount();
     });
 
     it('should include commandKind for file commands', async () => {
@@ -786,7 +929,7 @@ describe('useSlashCompletion', () => {
         },
       ] as SlashCommand[];
 
-      const { result } = renderHook(() =>
+      const { result, unmount } = renderHook(() =>
         useTestHarnessForSlashCompletion(
           true,
           '/custom',
@@ -804,11 +947,13 @@ describe('useSlashCompletion', () => {
             commandKind: CommandKind.FILE,
           },
         ]);
+        expect(result.current.completionStart).toBe(1);
       });
+      unmount();
     });
   });
 
-  it('should not call shared callbacks when disabled', () => {
+  it('should not call shared callbacks when disabled', async () => {
     const mockSetSuggestions = vi.fn();
     const mockSetIsLoadingSuggestions = vi.fn();
     const mockSetIsPerfectMatch = vi.fn();
@@ -820,7 +965,7 @@ describe('useSlashCompletion', () => {
       }),
     ];
 
-    const { rerender } = renderHook(
+    const { rerender, unmount } = renderHook(
       ({ enabled, query }) =>
         useSlashCompletion({
           enabled,
@@ -845,9 +990,17 @@ describe('useSlashCompletion', () => {
     rerender({ enabled: false, query: '@src/file.ts' });
     rerender({ enabled: false, query: '@src/file.tsx' });
 
+    // Wait for any internal async operations to settle to avoid act warnings
+    await act(async () => {
+      await new Promise((resolve) => setTimeout(resolve, 0));
+    });
+
     // Should not have called shared callbacks during @ completion typing
-    expect(mockSetSuggestions).not.toHaveBeenCalled();
-    expect(mockSetIsLoadingSuggestions).not.toHaveBeenCalled();
-    expect(mockSetIsPerfectMatch).not.toHaveBeenCalled();
+    await waitFor(() => {
+      expect(mockSetSuggestions).not.toHaveBeenCalled();
+      expect(mockSetIsLoadingSuggestions).not.toHaveBeenCalled();
+      expect(mockSetIsPerfectMatch).not.toHaveBeenCalled();
+    });
+    unmount();
   });
 });
