@@ -7,7 +7,8 @@
 /* eslint-disable @typescript-eslint/no-explicit-any */
 import type { Mock } from 'vitest';
 import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
-import { renderHook, act } from '@testing-library/react';
+import { act } from 'react';
+import { renderHook } from '../../test-utils/render.js';
 import {
   useReactToolScheduler,
   mapToDisplay,
@@ -31,13 +32,21 @@ import {
   ToolConfirmationOutcome,
   ApprovalMode,
   MockTool,
+  HookSystem,
 } from '@google/gemini-cli-core';
-import type { HistoryItemWithoutId, HistoryItemToolGroup } from '../types.js';
+import { createMockMessageBus } from '@google/gemini-cli-core/src/test-utils/mock-message-bus.js';
 import { ToolCallStatus } from '../types.js';
 
 // Mocks
 vi.mock('@google/gemini-cli-core', async () => {
-  const actual = await vi.importActual('@google/gemini-cli-core');
+  const actual = await vi.importActual<any>('@google/gemini-cli-core');
+  // Patch CoreToolScheduler to have cancelAll if it's missing in the test environment
+  if (
+    actual.CoreToolScheduler &&
+    !actual.CoreToolScheduler.prototype.cancelAll
+  ) {
+    actual.CoreToolScheduler.prototype.cancelAll = vi.fn();
+  }
   return {
     ...actual,
     ToolRegistry: vi.fn(),
@@ -67,13 +76,17 @@ const mockConfig = {
     authType: 'oauth-personal',
   }),
   getUseSmartEdit: () => false,
-  getUseModelRouter: () => false,
   getGeminiClient: () => null, // No client needed for these tests
   getShellExecutionConfig: () => ({ terminalWidth: 80, terminalHeight: 24 }),
   getEnableMessageBusIntegration: () => false,
   getMessageBus: () => null,
   getPolicyEngine: () => null,
+  isInteractive: () => false,
+  getExperiments: () => {},
+  getEnableHooks: () => false,
 } as unknown as Config;
+mockConfig.getMessageBus = vi.fn().mockReturnValue(createMockMessageBus());
+mockConfig.getHookSystem = vi.fn().mockReturnValue(new HookSystem(mockConfig));
 
 const mockTool = new MockTool({
   name: 'mockTool',
@@ -101,11 +114,9 @@ const mockToolRequiresConfirmation = new MockTool({
 
 describe('useReactToolScheduler in YOLO Mode', () => {
   let onComplete: Mock;
-  let setPendingHistoryItem: Mock;
 
   beforeEach(() => {
     onComplete = vi.fn();
-    setPendingHistoryItem = vi.fn();
     mockToolRegistry.getTool.mockClear();
     (mockToolRequiresConfirmation.execute as Mock).mockClear();
     (mockToolRequiresConfirmation.shouldConfirmExecute as Mock).mockClear();
@@ -128,8 +139,7 @@ describe('useReactToolScheduler in YOLO Mode', () => {
       useReactToolScheduler(
         onComplete,
         mockConfig as unknown as Config,
-        setPendingHistoryItem,
-        () => {},
+        () => undefined,
       ),
     );
 
@@ -154,13 +164,13 @@ describe('useReactToolScheduler in YOLO Mode', () => {
     });
 
     await act(async () => {
-      await vi.runAllTimersAsync(); // Process validation
+      await vi.advanceTimersByTimeAsync(0); // Process validation
     });
     await act(async () => {
-      await vi.runAllTimersAsync(); // Process scheduling
+      await vi.advanceTimersByTimeAsync(0); // Process scheduling
     });
     await act(async () => {
-      await vi.runAllTimersAsync(); // Process execution
+      await vi.advanceTimersByTimeAsync(0); // Process execution
     });
 
     // Check that execute WAS called
@@ -187,56 +197,40 @@ describe('useReactToolScheduler in YOLO Mode', () => {
         }),
       }),
     ]);
-
-    // Ensure no confirmation UI was triggered (setPendingHistoryItem should not have been called with confirmation details)
-    const setPendingHistoryItemCalls = setPendingHistoryItem.mock.calls;
-    const confirmationCall = setPendingHistoryItemCalls.find((call) => {
-      const item = typeof call[0] === 'function' ? call[0]({}) : call[0];
-      return item?.tools?.[0]?.confirmationDetails;
-    });
-    expect(confirmationCall).toBeUndefined();
   });
 });
 
 describe('useReactToolScheduler', () => {
-  // TODO(ntaylormullen): The following tests are skipped due to difficulties in
-  // reliably testing the asynchronous state updates and interactions with timers.
-  // These tests involve complex sequences of events, including confirmations,
-  // live output updates, and cancellations, which are challenging to assert
-  // correctly with the current testing setup. Further investigation is needed
-  // to find a robust way to test these scenarios.
   let onComplete: Mock;
-  let setPendingHistoryItem: Mock;
   let capturedOnConfirmForTest:
     | ((outcome: ToolConfirmationOutcome) => void | Promise<void>)
     | undefined;
 
+  const advanceAndSettle = async () => {
+    await act(async () => {
+      await vi.advanceTimersByTimeAsync(0);
+    });
+  };
+
+  const scheduleAndWaitForExecution = async (
+    schedule: (
+      req: ToolCallRequestInfo | ToolCallRequestInfo[],
+      signal: AbortSignal,
+    ) => void,
+    request: ToolCallRequestInfo | ToolCallRequestInfo[],
+  ) => {
+    act(() => {
+      schedule(request, new AbortController().signal);
+    });
+
+    await advanceAndSettle();
+    await advanceAndSettle();
+    await advanceAndSettle();
+  };
+
   beforeEach(() => {
     onComplete = vi.fn();
     capturedOnConfirmForTest = undefined;
-    setPendingHistoryItem = vi.fn((updaterOrValue) => {
-      let pendingItem: HistoryItemWithoutId | null = null;
-      if (typeof updaterOrValue === 'function') {
-        // Loosen the type for prevState to allow for more flexible updates in tests
-        const prevState: Partial<HistoryItemToolGroup> = {
-          type: 'tool_group', // Still default to tool_group for most cases
-          tools: [],
-        };
-
-        pendingItem = updaterOrValue(prevState as any); // Allow any for more flexibility
-      } else {
-        pendingItem = updaterOrValue;
-      }
-      // Capture onConfirm if it exists, regardless of the exact type of pendingItem
-      // This is a common pattern in these tests.
-      if (
-        (pendingItem as HistoryItemToolGroup)?.tools?.[0]?.confirmationDetails
-          ?.onConfirm
-      ) {
-        capturedOnConfirmForTest = (pendingItem as HistoryItemToolGroup)
-          .tools[0].confirmationDetails?.onConfirm;
-      }
-    });
 
     mockToolRegistry.getTool.mockClear();
     (mockTool.execute as Mock).mockClear();
@@ -273,8 +267,7 @@ describe('useReactToolScheduler', () => {
       useReactToolScheduler(
         onComplete,
         mockConfig as unknown as Config,
-        setPendingHistoryItem,
-        () => {},
+        () => undefined,
       ),
     );
 
@@ -292,133 +285,96 @@ describe('useReactToolScheduler', () => {
     (mockTool.shouldConfirmExecute as Mock).mockResolvedValue(null);
 
     const { result } = renderScheduler();
-    const schedule = result.current[1];
     const request: ToolCallRequestInfo = {
       callId: 'call1',
       name: 'mockTool',
       args: { param: 'value' },
     } as any;
 
-    act(() => {
-      schedule(request, new AbortController().signal);
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
+    let completedToolCalls: ToolCall[] = [];
+    onComplete.mockImplementation((calls) => {
+      completedToolCalls = calls;
     });
 
+    await scheduleAndWaitForExecution(result.current[1], request);
+
     expect(mockTool.execute).toHaveBeenCalledWith(request.args);
-    expect(onComplete).toHaveBeenCalledWith([
-      expect.objectContaining({
-        status: 'success',
-        request,
-        response: expect.objectContaining({
-          resultDisplay: 'Formatted tool output',
-          responseParts: [
-            {
-              functionResponse: {
-                id: 'call1',
-                name: 'mockTool',
-                response: { output: 'Tool output' },
-              },
-            },
-          ],
-        }),
-      }),
-    ]);
-    expect(result.current[0]).toEqual([]);
+    expect(completedToolCalls).toHaveLength(1);
+    expect(completedToolCalls[0].status).toBe('success');
+    expect(completedToolCalls[0].request).toBe(request);
+
+    if (
+      completedToolCalls[0].status === 'success' ||
+      completedToolCalls[0].status === 'error'
+    ) {
+      expect(completedToolCalls[0].response).toMatchSnapshot();
+    }
   });
 
-  it('should handle tool not found', async () => {
-    mockToolRegistry.getTool.mockReturnValue(undefined);
-    const { result } = renderScheduler();
-    const schedule = result.current[1];
-    const request: ToolCallRequestInfo = {
-      callId: 'call1',
-      name: 'nonexistentTool',
-      args: {},
-    } as any;
-
-    act(() => {
-      schedule(request, new AbortController().signal);
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-
-    expect(onComplete).toHaveBeenCalledWith([
-      expect.objectContaining({
-        status: 'error',
-        request,
-        response: expect.objectContaining({
-          error: expect.objectContaining({
-            message: expect.stringMatching(
-              /Tool "nonexistentTool" not found in registry/,
-            ),
-          }),
-        }),
-      }),
-    ]);
-    const errorMessage = onComplete.mock.calls[0][0][0].response.error.message;
-    expect(errorMessage).toContain('Did you mean one of:');
-    expect(errorMessage).toContain('"mockTool"');
-    expect(errorMessage).toContain('"anotherTool"');
-    expect(result.current[0]).toEqual([]);
-  });
-
-  it('should handle error during shouldConfirmExecute', async () => {
+  it('should clear previous tool calls when scheduling new ones', async () => {
     mockToolRegistry.getTool.mockReturnValue(mockTool);
-    const confirmError = new Error('Confirmation check failed');
-    (mockTool.shouldConfirmExecute as Mock).mockRejectedValue(confirmError);
+    (mockTool.execute as Mock).mockResolvedValue({
+      llmContent: 'Tool output',
+      returnDisplay: 'Formatted tool output',
+    } as ToolResult);
 
     const { result } = renderScheduler();
     const schedule = result.current[1];
-    const request: ToolCallRequestInfo = {
-      callId: 'call1',
+    const setToolCallsForDisplay = result.current[3];
+
+    // Manually set a tool call in the display.
+    const oldToolCall = {
+      request: { callId: 'oldCall' },
+      status: 'success',
+    } as any;
+    act(() => {
+      setToolCallsForDisplay([oldToolCall]);
+    });
+    expect(result.current[0]).toEqual([oldToolCall]);
+
+    const newRequest: ToolCallRequestInfo = {
+      callId: 'newCall',
       name: 'mockTool',
       args: {},
     } as any;
-
     act(() => {
-      schedule(request, new AbortController().signal);
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
+      schedule(newRequest, new AbortController().signal);
     });
 
-    expect(onComplete).toHaveBeenCalledWith([
-      expect.objectContaining({
-        status: 'error',
-        request,
-        response: expect.objectContaining({
-          error: confirmError,
-        }),
-      }),
-    ]);
-    expect(result.current[0]).toEqual([]);
+    // After scheduling, the old call should be gone,
+    // and the new one should be in the display in its initial state.
+    expect(result.current[0].length).toBe(1);
+    expect(result.current[0][0].request.callId).toBe('newCall');
+    expect(result.current[0][0].request.callId).not.toBe('oldCall');
+
+    // Let the new call finish.
+    await act(async () => {
+      await vi.advanceTimersByTimeAsync(0);
+    });
+    await act(async () => {
+      await vi.advanceTimersByTimeAsync(0);
+    });
+    await act(async () => {
+      await vi.advanceTimersByTimeAsync(0);
+    });
+    expect(onComplete).toHaveBeenCalled();
   });
 
-  it('should handle error during execute', async () => {
+  it('should cancel all running tool calls', async () => {
     mockToolRegistry.getTool.mockReturnValue(mockTool);
+
+    let resolveExecute: (value: ToolResult) => void = () => {};
+    const executePromise = new Promise<ToolResult>((resolve) => {
+      resolveExecute = resolve;
+    });
+    (mockTool.execute as Mock).mockReturnValue(executePromise);
     (mockTool.shouldConfirmExecute as Mock).mockResolvedValue(null);
-    const execError = new Error('Execution failed');
-    (mockTool.execute as Mock).mockRejectedValue(execError);
 
     const { result } = renderScheduler();
     const schedule = result.current[1];
+    const cancelAllToolCalls = result.current[4];
     const request: ToolCallRequestInfo = {
-      callId: 'call1',
+      callId: 'cancelCall',
       name: 'mockTool',
       args: {},
     } as any;
@@ -427,28 +383,116 @@ describe('useReactToolScheduler', () => {
       schedule(request, new AbortController().signal);
     });
     await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+      await vi.advanceTimersByTimeAsync(0);
+    }); // validation
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0); // Process scheduling
     });
+
+    // At this point, the tool is 'executing' and waiting on the promise.
+    expect(result.current[0][0].status).toBe('executing');
+
+    const cancelController = new AbortController();
+    act(() => {
+      cancelAllToolCalls(cancelController.signal);
+    });
+
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
     });
 
     expect(onComplete).toHaveBeenCalledWith([
       expect.objectContaining({
-        status: 'error',
+        status: 'cancelled',
         request,
-        response: expect.objectContaining({
-          error: execError,
-        }),
       }),
     ]);
-    expect(result.current[0]).toEqual([]);
+
+    // Clean up the pending promise to avoid open handles.
+    await act(async () => {
+      resolveExecute({ llmContent: 'output', returnDisplay: 'display' });
+    });
   });
 
-  it.skip('should handle tool requiring confirmation - approved', async () => {
+  it.each([
+    {
+      desc: 'tool not found',
+      setup: () => {
+        mockToolRegistry.getTool.mockReturnValue(undefined);
+      },
+      request: {
+        callId: 'call1',
+        name: 'nonexistentTool',
+        args: {},
+      } as any,
+      expectedErrorContains: [
+        'Tool "nonexistentTool" not found in registry',
+        'Did you mean one of:',
+      ],
+    },
+    {
+      desc: 'error during shouldConfirmExecute',
+      setup: () => {
+        mockToolRegistry.getTool.mockReturnValue(mockTool);
+        const confirmError = new Error('Confirmation check failed');
+        (mockTool.shouldConfirmExecute as Mock).mockRejectedValue(confirmError);
+      },
+      request: {
+        callId: 'call1',
+        name: 'mockTool',
+        args: {},
+      } as any,
+      expectedError: new Error('Confirmation check failed'),
+    },
+    {
+      desc: 'error during execute',
+      setup: () => {
+        mockToolRegistry.getTool.mockReturnValue(mockTool);
+        (mockTool.shouldConfirmExecute as Mock).mockResolvedValue(null);
+        const execError = new Error('Execution failed');
+        (mockTool.execute as Mock).mockRejectedValue(execError);
+      },
+      request: {
+        callId: 'call1',
+        name: 'mockTool',
+        args: {},
+      } as any,
+      expectedError: new Error('Execution failed'),
+    },
+  ])(
+    'should handle $desc',
+    async ({ setup, request, expectedErrorContains, expectedError }) => {
+      setup();
+      const { result } = renderScheduler();
+
+      let completedToolCalls: ToolCall[] = [];
+      onComplete.mockImplementation((calls) => {
+        completedToolCalls = calls;
+      });
+
+      await scheduleAndWaitForExecution(result.current[1], request);
+
+      expect(completedToolCalls).toHaveLength(1);
+      expect(completedToolCalls[0].status).toBe('error');
+      expect(completedToolCalls[0].request).toBe(request);
+
+      if (expectedErrorContains) {
+        expectedErrorContains.forEach((errorText) => {
+          expect(
+            (completedToolCalls[0] as any).response.error.message,
+          ).toContain(errorText);
+        });
+      }
+
+      if (expectedError) {
+        expect((completedToolCalls[0] as any).response.error.message).toBe(
+          expectedError.message,
+        );
+      }
+    },
+  );
+
+  it('should handle tool requiring confirmation - approved', async () => {
     mockToolRegistry.getTool.mockReturnValue(mockToolRequiresConfirmation);
     const expectedOutput = 'Confirmed output';
     (mockToolRequiresConfirmation.execute as Mock).mockResolvedValue({
@@ -467,50 +511,38 @@ describe('useReactToolScheduler', () => {
     act(() => {
       schedule(request, new AbortController().signal);
     });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
 
-    expect(setPendingHistoryItem).toHaveBeenCalled();
+    const waitingCall = result.current[0][0] as any;
+    expect(waitingCall.status).toBe('awaiting_approval');
+    capturedOnConfirmForTest = waitingCall.confirmationDetails?.onConfirm;
     expect(capturedOnConfirmForTest).toBeDefined();
 
     await act(async () => {
       await capturedOnConfirmForTest?.(ToolConfirmationOutcome.ProceedOnce);
     });
 
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
+    await advanceAndSettle();
+    await advanceAndSettle();
 
     expect(mockOnUserConfirmForToolConfirmation).toHaveBeenCalledWith(
       ToolConfirmationOutcome.ProceedOnce,
     );
     expect(mockToolRequiresConfirmation.execute).toHaveBeenCalled();
-    expect(onComplete).toHaveBeenCalledWith([
-      expect.objectContaining({
-        status: 'success',
-        request,
-        response: expect.objectContaining({
-          resultDisplay: 'Confirmed display',
-          responseParts: expect.arrayContaining([
-            expect.objectContaining({
-              functionResponse: expect.objectContaining({
-                response: { output: expectedOutput },
-              }),
-            }),
-          ]),
-        }),
-      }),
-    ]);
+
+    const completedCalls = onComplete.mock.calls[0][0] as ToolCall[];
+    expect(completedCalls[0].status).toBe('success');
+    expect(completedCalls[0].request).toBe(request);
+    if (
+      completedCalls[0].status === 'success' ||
+      completedCalls[0].status === 'error'
+    ) {
+      expect(completedCalls[0].response).toMatchSnapshot();
+    }
   });
 
-  it.skip('should handle tool requiring confirmation - cancelled by user', async () => {
+  it('should handle tool requiring confirmation - cancelled by user', async () => {
     mockToolRegistry.getTool.mockReturnValue(mockToolRequiresConfirmation);
     const { result } = renderScheduler();
     const schedule = result.current[1];
@@ -523,46 +555,36 @@ describe('useReactToolScheduler', () => {
     act(() => {
       schedule(request, new AbortController().signal);
     });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
 
-    expect(setPendingHistoryItem).toHaveBeenCalled();
+    const waitingCall = result.current[0][0] as any;
+    expect(waitingCall.status).toBe('awaiting_approval');
+    capturedOnConfirmForTest = waitingCall.confirmationDetails?.onConfirm;
     expect(capturedOnConfirmForTest).toBeDefined();
 
     await act(async () => {
       await capturedOnConfirmForTest?.(ToolConfirmationOutcome.Cancel);
     });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
+    await advanceAndSettle();
 
     expect(mockOnUserConfirmForToolConfirmation).toHaveBeenCalledWith(
       ToolConfirmationOutcome.Cancel,
     );
-    expect(onComplete).toHaveBeenCalledWith([
-      expect.objectContaining({
-        status: 'cancelled',
-        request,
-        response: expect.objectContaining({
-          responseParts: expect.arrayContaining([
-            expect.objectContaining({
-              functionResponse: expect.objectContaining({
-                response: expect.objectContaining({
-                  error: `User did not allow tool call ${request.name}. Reason: User cancelled.`,
-                }),
-              }),
-            }),
-          ]),
-        }),
-      }),
-    ]);
+
+    const completedCalls = onComplete.mock.calls[0][0] as ToolCall[];
+    expect(completedCalls[0].status).toBe('cancelled');
+    expect(completedCalls[0].request).toBe(request);
+    if (
+      completedCalls[0].status === 'success' ||
+      completedCalls[0].status === 'error' ||
+      completedCalls[0].status === 'cancelled'
+    ) {
+      expect(completedCalls[0].response).toMatchSnapshot();
+    }
   });
 
-  it.skip('should handle live output updates', async () => {
+  it('should handle live output updates', async () => {
     mockToolRegistry.getTool.mockReturnValue(mockToolWithLiveOutput);
     let liveUpdateFn: ((output: string) => void) | undefined;
     let resolveExecutePromise: (value: ToolResult) => void;
@@ -585,7 +607,6 @@ describe('useReactToolScheduler', () => {
     );
 
     const { result } = renderScheduler();
-    const schedule = result.current[1];
     const request: ToolCallRequestInfo = {
       callId: 'liveCall',
       name: 'mockToolWithLiveOutput',
@@ -593,28 +614,22 @@ describe('useReactToolScheduler', () => {
     } as any;
 
     act(() => {
-      schedule(request, new AbortController().signal);
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
+      result.current[1](request, new AbortController().signal);
     });
+    await advanceAndSettle();
 
     expect(liveUpdateFn).toBeDefined();
-    expect(setPendingHistoryItem).toHaveBeenCalled();
+    expect(result.current[0][0].status).toBe('executing');
 
     await act(async () => {
       liveUpdateFn?.('Live output 1');
     });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
 
     await act(async () => {
       liveUpdateFn?.('Live output 2');
     });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
 
     act(() => {
       resolveExecutePromise({
@@ -622,29 +637,18 @@ describe('useReactToolScheduler', () => {
         returnDisplay: 'Final display',
       } as ToolResult);
     });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
-    await act(async () => {
-      await vi.runAllTimersAsync();
-    });
+    await advanceAndSettle();
+    await advanceAndSettle();
 
-    expect(onComplete).toHaveBeenCalledWith([
-      expect.objectContaining({
-        status: 'success',
-        request,
-        response: expect.objectContaining({
-          resultDisplay: 'Final display',
-          responseParts: expect.arrayContaining([
-            expect.objectContaining({
-              functionResponse: expect.objectContaining({
-                response: { output: 'Final output' },
-              }),
-            }),
-          ]),
-        }),
-      }),
-    ]);
+    const completedCalls = onComplete.mock.calls[0][0] as ToolCall[];
+    expect(completedCalls[0].status).toBe('success');
+    expect(completedCalls[0].request).toBe(request);
+    if (
+      completedCalls[0].status === 'success' ||
+      completedCalls[0].status === 'error'
+    ) {
+      expect(completedCalls[0].response).toMatchSnapshot();
+    }
     expect(result.current[0]).toEqual([]);
   });
 
@@ -684,16 +688,16 @@ describe('useReactToolScheduler', () => {
       schedule(requests, new AbortController().signal);
     });
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
     });
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
     });
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
     });
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
     });
 
     expect(onComplete).toHaveBeenCalledTimes(1);
@@ -739,10 +743,12 @@ describe('useReactToolScheduler', () => {
         ],
       }),
     });
-    expect(result.current[0]).toEqual([]);
+
+    expect(completedCalls).toHaveLength(2);
+    expect(completedCalls.every((t) => t.status === 'success')).toBe(true);
   });
 
-  it.skip('should throw error if scheduling while already running', async () => {
+  it('should queue if scheduling while already running', async () => {
     mockToolRegistry.getTool.mockReturnValue(mockTool);
     const longExecutePromise = new Promise<ToolResult>((resolve) =>
       setTimeout(
@@ -774,18 +780,18 @@ describe('useReactToolScheduler', () => {
       schedule(request1, new AbortController().signal);
     });
     await act(async () => {
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
     });
 
-    expect(() => schedule(request2, new AbortController().signal)).toThrow(
-      'Cannot schedule tool calls while other tool calls are running',
-    );
+    act(() => {
+      schedule(request2, new AbortController().signal);
+    });
 
     await act(async () => {
       await vi.advanceTimersByTimeAsync(50);
-      await vi.runAllTimersAsync();
+      await vi.advanceTimersByTimeAsync(0);
       await act(async () => {
-        await vi.runAllTimersAsync();
+        await vi.advanceTimersByTimeAsync(0);
       });
     });
     expect(onComplete).toHaveBeenCalledWith([
@@ -795,7 +801,22 @@ describe('useReactToolScheduler', () => {
         response: expect.objectContaining({ resultDisplay: 'done display' }),
       }),
     ]);
-    expect(result.current[0]).toEqual([]);
+    await act(async () => {
+      await vi.advanceTimersByTimeAsync(50);
+      await vi.advanceTimersByTimeAsync(0);
+      await act(async () => {
+        await vi.advanceTimersByTimeAsync(0);
+      });
+    });
+    expect(onComplete).toHaveBeenCalledWith([
+      expect.objectContaining({
+        status: 'success',
+        request: request2,
+        response: expect.objectContaining({ resultDisplay: 'done display' }),
+      }),
+    ]);
+    const toolCalls = result.current[0];
+    expect(toolCalls).toHaveLength(0);
   });
 });
 
