@@ -5,6 +5,7 @@
  */
 
 import { type ColorsTheme, Theme } from './theme.js';
+import { interpolateColor } from './color-utils.js';
 
 const draculaColors: ColorsTheme = {
   type: 'dark',
@@ -21,6 +22,7 @@ const draculaColors: ColorsTheme = {
   DiffRemoved: '#6e1818',
   Comment: '#6272a4',
   Gray: '#6272a4',
+  DarkGray: interpolateColor('#6272a4', '#282a36', 0.5),
   GradientColors: ['#ff79c6', '#8be9fd'],
 };
 
