@@ -5,6 +5,7 @@
  */
 
 import { type ColorsTheme, Theme, lightTheme } from './theme.js';
+import { interpolateColor } from './color-utils.js';
 
 const googleCodeColors: ColorsTheme = {
   type: 'light',
@@ -21,6 +22,7 @@ const googleCodeColors: ColorsTheme = {
   DiffRemoved: '#FEDEDE',
   Comment: '#5f6368',
   Gray: lightTheme.Gray,
+  DarkGray: interpolateColor(lightTheme.Gray, '#ffffff', 0.5),
   GradientColors: ['#066', '#606'],
 };
 
