@@ -7,7 +7,11 @@
 import { describe, it, expect, vi, beforeEach, afterEach } from 'vitest';
 import * as fs from 'node:fs/promises';
 import * as path from 'node:path';
-import { SESSION_FILE_PREFIX, type Config } from '@google/gemini-cli-core';
+import {
+  SESSION_FILE_PREFIX,
+  type Config,
+  debugLogger,
+} from '@google/gemini-cli-core';
 import type { Settings } from '../config/settings.js';
 import { cleanupExpiredSessions } from './sessionCleanup.js';
 import { type SessionInfo, getAllSessionFiles } from './sessionUtils.js';
@@ -44,27 +48,51 @@ function createTestSessions(): SessionInfo[] {
   return [
     {
       id: 'current123',
+      file: `${SESSION_FILE_PREFIX}2025-01-20T10-30-00-current12`,
       fileName: `${SESSION_FILE_PREFIX}2025-01-20T10-30-00-current12.json`,
+      startTime: now.toISOString(),
       lastUpdated: now.toISOString(),
+      messageCount: 5,
+      displayName: 'Current session',
+      firstUserMessage: 'Current session',
       isCurrentSession: true,
+      index: 1,
     },
     {
       id: 'recent456',
+      file: `${SESSION_FILE_PREFIX}2025-01-18T15-45-00-recent45`,
       fileName: `${SESSION_FILE_PREFIX}2025-01-18T15-45-00-recent45.json`,
+      startTime: oneWeekAgo.toISOString(),
       lastUpdated: oneWeekAgo.toISOString(),
+      messageCount: 10,
+      displayName: 'Recent session',
+      firstUserMessage: 'Recent session',
       isCurrentSession: false,
+      index: 2,
     },
     {
       id: 'old789abc',
+      file: `${SESSION_FILE_PREFIX}2025-01-10T09-15-00-old789ab`,
       fileName: `${SESSION_FILE_PREFIX}2025-01-10T09-15-00-old789ab.json`,
+      startTime: twoWeeksAgo.toISOString(),
       lastUpdated: twoWeeksAgo.toISOString(),
+      messageCount: 3,
+      displayName: 'Old session',
+      firstUserMessage: 'Old session',
       isCurrentSession: false,
+      index: 3,
     },
     {
       id: 'ancient12',
+      file: `${SESSION_FILE_PREFIX}2024-12-25T12-00-00-ancient1`,
       fileName: `${SESSION_FILE_PREFIX}2024-12-25T12-00-00-ancient1.json`,
+      startTime: oneMonthAgo.toISOString(),
       lastUpdated: oneMonthAgo.toISOString(),
+      messageCount: 15,
+      displayName: 'Ancient session',
+      firstUserMessage: 'Ancient session',
       isCurrentSession: false,
+      index: 4,
     },
   ];
 }
@@ -373,7 +401,9 @@ describe('Session Cleanup', () => {
       );
       mockFs.unlink.mockResolvedValue(undefined);
 
-      const debugSpy = vi.spyOn(console, 'debug').mockImplementation(() => {});
+      const debugSpy = vi
+        .spyOn(debugLogger, 'debug')
+        .mockImplementation(() => {});
 
       await cleanupExpiredSessions(config, settings);
 
@@ -409,27 +439,51 @@ describe('Session Cleanup', () => {
       const testSessions: SessionInfo[] = [
         {
           id: 'current',
+          file: `${SESSION_FILE_PREFIX}current`,
           fileName: `${SESSION_FILE_PREFIX}current.json`,
+          startTime: now.toISOString(),
           lastUpdated: now.toISOString(),
+          messageCount: 1,
+          displayName: 'Current',
+          firstUserMessage: 'Current',
           isCurrentSession: true,
+          index: 1,
         },
         {
           id: 'session5d',
+          file: `${SESSION_FILE_PREFIX}5d`,
           fileName: `${SESSION_FILE_PREFIX}5d.json`,
+          startTime: fiveDaysAgo.toISOString(),
           lastUpdated: fiveDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '5 days old',
+          firstUserMessage: '5 days',
           isCurrentSession: false,
+          index: 2,
         },
         {
           id: 'session8d',
+          file: `${SESSION_FILE_PREFIX}8d`,
           fileName: `${SESSION_FILE_PREFIX}8d.json`,
+          startTime: eightDaysAgo.toISOString(),
           lastUpdated: eightDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '8 days old',
+          firstUserMessage: '8 days',
           isCurrentSession: false,
+          index: 3,
         },
         {
           id: 'session15d',
+          file: `${SESSION_FILE_PREFIX}15d`,
           fileName: `${SESSION_FILE_PREFIX}15d.json`,
+          startTime: fifteenDaysAgo.toISOString(),
           lastUpdated: fifteenDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '15 days old',
+          firstUserMessage: '15 days',
           isCurrentSession: false,
+          index: 4,
         },
       ];
 
@@ -507,27 +561,51 @@ describe('Session Cleanup', () => {
       const testSessions: SessionInfo[] = [
         {
           id: 'current',
+          file: `${SESSION_FILE_PREFIX}current`,
           fileName: `${SESSION_FILE_PREFIX}current.json`,
+          startTime: now.toISOString(),
           lastUpdated: now.toISOString(),
+          messageCount: 1,
+          displayName: 'Current',
+          firstUserMessage: 'Current',
           isCurrentSession: true,
+          index: 1,
         },
         {
           id: 'session1d',
+          file: `${SESSION_FILE_PREFIX}1d`,
           fileName: `${SESSION_FILE_PREFIX}1d.json`,
+          startTime: oneDayAgo.toISOString(),
           lastUpdated: oneDayAgo.toISOString(),
+          messageCount: 1,
+          displayName: '1 day old',
+          firstUserMessage: '1 day',
           isCurrentSession: false,
+          index: 2,
         },
         {
           id: 'session7d',
+          file: `${SESSION_FILE_PREFIX}7d`,
           fileName: `${SESSION_FILE_PREFIX}7d.json`,
+          startTime: sevenDaysAgo.toISOString(),
           lastUpdated: sevenDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '7 days old',
+          firstUserMessage: '7 days',
           isCurrentSession: false,
+          index: 3,
         },
         {
           id: 'session13d',
+          file: `${SESSION_FILE_PREFIX}13d`,
           fileName: `${SESSION_FILE_PREFIX}13d.json`,
+          startTime: thirteenDaysAgo.toISOString(),
           lastUpdated: thirteenDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '13 days old',
+          firstUserMessage: '13 days',
           isCurrentSession: false,
+          index: 4,
         },
       ];
 
@@ -579,9 +657,15 @@ describe('Session Cleanup', () => {
       const sessions: SessionInfo[] = [
         {
           id: 'current',
+          file: `${SESSION_FILE_PREFIX}current`,
           fileName: `${SESSION_FILE_PREFIX}current.json`,
+          startTime: now.toISOString(),
           lastUpdated: now.toISOString(),
+          messageCount: 1,
+          displayName: 'Current (newest)',
+          firstUserMessage: 'Current',
           isCurrentSession: true,
+          index: 1,
         },
       ];
 
@@ -590,9 +674,15 @@ describe('Session Cleanup', () => {
         const daysAgo = new Date(now.getTime() - i * 24 * 60 * 60 * 1000);
         sessions.push({
           id: `session${i}`,
+          file: `${SESSION_FILE_PREFIX}${i}d`,
           fileName: `${SESSION_FILE_PREFIX}${i}d.json`,
+          startTime: daysAgo.toISOString(),
           lastUpdated: daysAgo.toISOString(),
+          messageCount: 1,
+          displayName: `${i} days old`,
+          firstUserMessage: `${i} days`,
           isCurrentSession: false,
+          index: i + 1,
         });
       }
 
@@ -693,33 +783,63 @@ describe('Session Cleanup', () => {
       const testSessions: SessionInfo[] = [
         {
           id: 'current',
+          file: `${SESSION_FILE_PREFIX}current`,
           fileName: `${SESSION_FILE_PREFIX}current.json`,
+          startTime: now.toISOString(),
           lastUpdated: now.toISOString(),
+          messageCount: 1,
+          displayName: 'Current',
+          firstUserMessage: 'Current',
           isCurrentSession: true,
+          index: 1,
         },
         {
           id: 'session3d',
+          file: `${SESSION_FILE_PREFIX}3d`,
           fileName: `${SESSION_FILE_PREFIX}3d.json`,
+          startTime: threeDaysAgo.toISOString(),
           lastUpdated: threeDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '3 days old',
+          firstUserMessage: '3 days',
           isCurrentSession: false,
+          index: 2,
         },
         {
           id: 'session5d',
+          file: `${SESSION_FILE_PREFIX}5d`,
           fileName: `${SESSION_FILE_PREFIX}5d.json`,
+          startTime: fiveDaysAgo.toISOString(),
           lastUpdated: fiveDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '5 days old',
+          firstUserMessage: '5 days',
           isCurrentSession: false,
+          index: 3,
         },
         {
           id: 'session7d',
+          file: `${SESSION_FILE_PREFIX}7d`,
           fileName: `${SESSION_FILE_PREFIX}7d.json`,
+          startTime: sevenDaysAgo.toISOString(),
           lastUpdated: sevenDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '7 days old',
+          firstUserMessage: '7 days',
           isCurrentSession: false,
+          index: 4,
         },
         {
           id: 'session12d',
+          file: `${SESSION_FILE_PREFIX}12d`,
           fileName: `${SESSION_FILE_PREFIX}12d.json`,
+          startTime: twelveDaysAgo.toISOString(),
           lastUpdated: twelveDaysAgo.toISOString(),
+          messageCount: 1,
+          displayName: '12 days old',
+          firstUserMessage: '12 days',
           isCurrentSession: false,
+          index: 5,
         },
       ];
 
