@@ -5,11 +5,17 @@
  */
 
 import type { Config } from '@google/gemini-cli-core';
-import { AuthType, debugLogger, OutputFormat } from '@google/gemini-cli-core';
+import {
+  AuthType,
+  debugLogger,
+  OutputFormat,
+  ExitCodes,
+} from '@google/gemini-cli-core';
 import { USER_SETTINGS_PATH } from './config/settings.js';
 import { validateAuthMethod } from './config/auth.js';
 import { type LoadedSettings } from './config/settings.js';
 import { handleError } from './utils/errors.js';
+import { runExitCleanup } from './utils/cleanup.js';
 
 function getAuthTypeFromEnv(): AuthType | undefined {
   if (process.env['GOOGLE_GENAI_USE_GCA'] === 'true') {
@@ -62,11 +68,12 @@ export async function validateNonInteractiveAuth(
       handleError(
         error instanceof Error ? error : new Error(String(error)),
         nonInteractiveConfig,
-        1,
+        ExitCodes.FATAL_AUTHENTICATION_ERROR,
       );
     } else {
       debugLogger.error(error instanceof Error ? error.message : String(error));
-      process.exit(1);
+      await runExitCleanup();
+      process.exit(ExitCodes.FATAL_AUTHENTICATION_ERROR);
     }
   }
 }
