@@ -6,18 +6,28 @@
 
 /// <reference types="vitest" />
 import { defineConfig } from 'vitest/config';
+import { fileURLToPath } from 'node:url';
+import * as path from 'node:path';
+
+const __dirname = path.dirname(fileURLToPath(import.meta.url));
 
 export default defineConfig({
+  resolve: {
+    conditions: ['test'],
+  },
   test: {
-    include: ['**/*.{test,spec}.?(c|m)[jt]s?(x)', 'config.test.ts'],
+    include: ['**/*.{test,spec}.{js,ts,jsx,tsx}', 'config.test.ts'],
     exclude: ['**/node_modules/**', '**/dist/**', '**/cypress/**'],
-    environment: 'jsdom',
+    environment: 'node',
     globals: true,
     reporters: ['default', 'junit'],
-    silent: true,
+
     outputFile: {
       junit: 'junit.xml',
     },
+    alias: {
+      react: path.resolve(__dirname, '../../node_modules/react'),
+    },
     setupFiles: ['./test-setup.ts'],
     coverage: {
       enabled: true,
