@@ -47,6 +47,13 @@ describe('ide-installer', () => {
       expect(installer).not.toBeNull();
       expect(installer?.install).toEqual(expect.any(Function));
     });
+
+    it('returns an AntigravityInstaller for "antigravity"', () => {
+      const installer = getIdeInstaller(IDE_DEFINITIONS.antigravity);
+
+      expect(installer).not.toBeNull();
+      expect(installer?.install).toEqual(expect.any(Function));
+    });
   });
 
   describe('VsCodeInstaller', () => {
@@ -188,3 +195,59 @@ describe('ide-installer', () => {
     });
   });
 });
+
+describe('AntigravityInstaller', () => {
+  function setup({
+    execSync = () => '',
+    platform = 'linux' as NodeJS.Platform,
+  }: {
+    execSync?: () => string;
+    platform?: NodeJS.Platform;
+  } = {}) {
+    vi.spyOn(child_process, 'execSync').mockImplementation(execSync);
+    const installer = getIdeInstaller(IDE_DEFINITIONS.antigravity, platform)!;
+
+    return { installer };
+  }
+
+  it('installs the extension using the alias', async () => {
+    vi.stubEnv('ANTIGRAVITY_CLI_ALIAS', 'agy');
+    const { installer } = setup({});
+    const result = await installer.install();
+
+    expect(result.success).toBe(true);
+    expect(child_process.spawnSync).toHaveBeenCalledWith(
+      'agy',
+      [
+        '--install-extension',
+        'google.gemini-cli-vscode-ide-companion',
+        '--force',
+      ],
+      { stdio: 'pipe', shell: false },
+    );
+  });
+
+  it('returns a failure message if the alias is not set', async () => {
+    vi.stubEnv('ANTIGRAVITY_CLI_ALIAS', '');
+    const { installer } = setup({});
+    const result = await installer.install();
+
+    expect(result.success).toBe(false);
+    expect(result.message).toContain(
+      'ANTIGRAVITY_CLI_ALIAS environment variable not set',
+    );
+  });
+
+  it('returns a failure message if the command is not found', async () => {
+    vi.stubEnv('ANTIGRAVITY_CLI_ALIAS', 'not-a-command');
+    const { installer } = setup({
+      execSync: () => {
+        throw new Error('Command not found');
+      },
+    });
+    const result = await installer.install();
+
+    expect(result.success).toBe(false);
+    expect(result.message).toContain('not-a-command not found');
+  });
+});
