@@ -58,12 +58,11 @@ describe('FileTokenStorage', () => {
   });
 
   describe('getCredentials', () => {
-    it('should throw error when file does not exist', async () => {
+    it('should return null when file does not exist', async () => {
       mockFs.readFile.mockRejectedValue({ code: 'ENOENT' });
 
-      await expect(storage.getCredentials('test-server')).rejects.toThrow(
-        'Token file does not exist',
-      );
+      const result = await storage.getCredentials('test-server');
+      expect(result).toBeNull();
     });
 
     it('should return null for expired tokens', async () => {
@@ -179,7 +178,7 @@ describe('FileTokenStorage', () => {
       mockFs.readFile.mockRejectedValue({ code: 'ENOENT' });
 
       await expect(storage.deleteCredentials('test-server')).rejects.toThrow(
-        'Token file does not exist',
+        'No credentials found for test-server',
       );
     });
 
@@ -246,12 +245,11 @@ describe('FileTokenStorage', () => {
   });
 
   describe('listServers', () => {
-    it('should throw error when file does not exist', async () => {
+    it('should return empty list when file does not exist', async () => {
       mockFs.readFile.mockRejectedValue({ code: 'ENOENT' });
 
-      await expect(storage.listServers()).rejects.toThrow(
-        'Token file does not exist',
-      );
+      const result = await storage.listServers();
+      expect(result).toEqual([]);
     });
 
     it('should return list of server names', async () => {
