@@ -9,9 +9,18 @@ import type { SessionMetrics } from '../telemetry/uiTelemetry.js';
 import type { JsonError, JsonOutput } from './types.js';
 
 export class JsonFormatter {
-  format(response?: string, stats?: SessionMetrics, error?: JsonError): string {
+  format(
+    sessionId?: string,
+    response?: string,
+    stats?: SessionMetrics,
+    error?: JsonError,
+  ): string {
     const output: JsonOutput = {};
 
+    if (sessionId) {
+      output.session_id = sessionId;
+    }
+
     if (response !== undefined) {
       output.response = stripAnsi(response);
     }
@@ -27,13 +36,17 @@ export class JsonFormatter {
     return JSON.stringify(output, null, 2);
   }
 
-  formatError(error: Error, code?: string | number): string {
+  formatError(
+    error: Error,
+    code?: string | number,
+    sessionId?: string,
+  ): string {
     const jsonError: JsonError = {
       type: error.constructor.name,
       message: stripAnsi(error.message),
       ...(code && { code }),
     };
 
-    return this.format(undefined, undefined, jsonError);
+    return this.format(sessionId, undefined, undefined, jsonError);
   }
 }
