@@ -13,15 +13,11 @@ import type {
   RoutingStrategy,
 } from '../routingStrategy.js';
 import {
-  DEFAULT_GEMINI_FLASH_MODEL,
-  DEFAULT_GEMINI_FLASH_LITE_MODEL,
-  DEFAULT_GEMINI_MODEL,
+  GEMINI_MODEL_ALIAS_FLASH,
+  GEMINI_MODEL_ALIAS_PRO,
+  resolveModel,
 } from '../../config/models.js';
-import {
-  type GenerateContentConfig,
-  createUserContent,
-  Type,
-} from '@google/genai';
+import { createUserContent, Type } from '@google/genai';
 import type { Config } from '../../config/config.js';
 import {
   isFunctionCall,
@@ -29,14 +25,6 @@ import {
 } from '../../utils/messageInspectors.js';
 import { debugLogger } from '../../utils/debugLogger.js';
 
-const CLASSIFIER_GENERATION_CONFIG: GenerateContentConfig = {
-  temperature: 0,
-  maxOutputTokens: 1024,
-  thinkingConfig: {
-    thinkingBudget: 512, // This counts towards output max, so we don't want -1.
-  },
-};
-
 // The number of recent history turns to provide to the router for context.
 const HISTORY_TURNS_FOR_CONTEXT = 4;
 const HISTORY_SEARCH_WINDOW = 20;
@@ -144,7 +132,7 @@ export class ClassifierStrategy implements RoutingStrategy {
 
   async route(
     context: RoutingContext,
-    _config: Config,
+    config: Config,
     baseLlmClient: BaseLlmClient,
   ): Promise<RoutingDecision | null> {
     const startTime = Date.now();
@@ -171,11 +159,10 @@ export class ClassifierStrategy implements RoutingStrategy {
       const finalHistory = cleanHistory.slice(-HISTORY_TURNS_FOR_CONTEXT);
 
       const jsonResponse = await baseLlmClient.generateJson({
+        modelConfigKey: { model: 'classifier' },
         contents: [...finalHistory, createUserContent(context.request)],
         schema: RESPONSE_SCHEMA,
-        model: DEFAULT_GEMINI_FLASH_LITE_MODEL,
         systemInstruction: CLASSIFIER_SYSTEM_PROMPT,
-        config: CLASSIFIER_GENERATION_CONFIG,
         abortSignal: context.signal,
         promptId,
       });
@@ -187,7 +174,10 @@ export class ClassifierStrategy implements RoutingStrategy {
 
       if (routerResponse.model_choice === FLASH_MODEL) {
         return {
-          model: DEFAULT_GEMINI_FLASH_MODEL,
+          model: resolveModel(
+            GEMINI_MODEL_ALIAS_FLASH,
+            config.getPreviewFeatures(),
+          ),
           metadata: {
             source: 'Classifier',
             latencyMs,
@@ -196,7 +186,10 @@ export class ClassifierStrategy implements RoutingStrategy {
         };
       } else {
         return {
-          model: DEFAULT_GEMINI_MODEL,
+          model: resolveModel(
+            GEMINI_MODEL_ALIAS_PRO,
+            config.getPreviewFeatures(),
+          ),
           metadata: {
             source: 'Classifier',
             reasoning,
