@@ -18,13 +18,13 @@ export interface FilterFilesOptions {
 
 export interface FilterReport {
   filteredPaths: string[];
-  gitIgnoredCount: number;
-  geminiIgnoredCount: number;
+  ignoredCount: number;
 }
 
 export class FileDiscoveryService {
   private gitIgnoreFilter: GitIgnoreFilter | null = null;
   private geminiIgnoreFilter: GeminiIgnoreFilter | null = null;
+  private combinedIgnoreFilter: GitIgnoreFilter | null = null;
   private projectRoot: string;
 
   constructor(projectRoot: string) {
@@ -33,26 +33,35 @@ export class FileDiscoveryService {
       this.gitIgnoreFilter = new GitIgnoreParser(this.projectRoot);
     }
     this.geminiIgnoreFilter = new GeminiIgnoreParser(this.projectRoot);
+
+    if (this.gitIgnoreFilter) {
+      const geminiPatterns = this.geminiIgnoreFilter.getPatterns();
+      // Create combined parser: .gitignore + .geminiignore
+      this.combinedIgnoreFilter = new GitIgnoreParser(
+        this.projectRoot,
+        geminiPatterns,
+      );
+    }
   }
 
   /**
    * Filters a list of file paths based on git ignore rules
    */
-  filterFiles(
-    filePaths: string[],
-    options: FilterFilesOptions = {
-      respectGitIgnore: true,
-      respectGeminiIgnore: true,
-    },
-  ): string[] {
+  filterFiles(filePaths: string[], options: FilterFilesOptions = {}): string[] {
+    const { respectGitIgnore = true, respectGeminiIgnore = true } = options;
     return filePaths.filter((filePath) => {
-      if (options.respectGitIgnore && this.shouldGitIgnoreFile(filePath)) {
+      if (
+        respectGitIgnore &&
+        respectGeminiIgnore &&
+        this.combinedIgnoreFilter
+      ) {
+        return !this.combinedIgnoreFilter.isIgnored(filePath);
+      }
+
+      if (respectGitIgnore && this.gitIgnoreFilter?.isIgnored(filePath)) {
         return false;
       }
-      if (
-        options.respectGeminiIgnore &&
-        this.shouldGeminiIgnoreFile(filePath)
-      ) {
+      if (respectGeminiIgnore && this.geminiIgnoreFilter?.isIgnored(filePath)) {
         return false;
       }
       return true;
@@ -70,51 +79,15 @@ export class FileDiscoveryService {
       respectGeminiIgnore: true,
     },
   ): FilterReport {
-    const filteredPaths: string[] = [];
-    let gitIgnoredCount = 0;
-    let geminiIgnoredCount = 0;
-
-    for (const filePath of filePaths) {
-      if (opts.respectGitIgnore && this.shouldGitIgnoreFile(filePath)) {
-        gitIgnoredCount++;
-        continue;
-      }
-
-      if (opts.respectGeminiIgnore && this.shouldGeminiIgnoreFile(filePath)) {
-        geminiIgnoredCount++;
-        continue;
-      }
-
-      filteredPaths.push(filePath);
-    }
+    const filteredPaths = this.filterFiles(filePaths, opts);
+    const ignoredCount = filePaths.length - filteredPaths.length;
 
     return {
       filteredPaths,
-      gitIgnoredCount,
-      geminiIgnoredCount,
+      ignoredCount,
     };
   }
 
-  /**
-   * Checks if a single file should be git-ignored
-   */
-  shouldGitIgnoreFile(filePath: string): boolean {
-    if (this.gitIgnoreFilter) {
-      return this.gitIgnoreFilter.isIgnored(filePath);
-    }
-    return false;
-  }
-
-  /**
-   * Checks if a single file should be gemini-ignored
-   */
-  shouldGeminiIgnoreFile(filePath: string): boolean {
-    if (this.geminiIgnoreFilter) {
-      return this.geminiIgnoreFilter.isIgnored(filePath);
-    }
-    return false;
-  }
-
   /**
    * Unified method to check if a file should be ignored based on filtering options
    */
@@ -122,21 +95,6 @@ export class FileDiscoveryService {
     filePath: string,
     options: FilterFilesOptions = {},
   ): boolean {
-    const { respectGitIgnore = true, respectGeminiIgnore = true } = options;
-
-    if (respectGitIgnore && this.shouldGitIgnoreFile(filePath)) {
-      return true;
-    }
-    if (respectGeminiIgnore && this.shouldGeminiIgnoreFile(filePath)) {
-      return true;
-    }
-    return false;
-  }
-
-  /**
-   * Returns loaded patterns from .geminiignore
-   */
-  getGeminiIgnorePatterns(): string[] {
-    return this.geminiIgnoreFilter?.getPatterns() ?? [];
+    return this.filterFiles([filePath], options).length === 0;
   }
 }
