@@ -92,7 +92,10 @@ describe('Telemetry Metrics', () => {
   let recordGenAiClientTokenUsageModule: typeof import('./metrics.js').recordGenAiClientTokenUsage;
   let recordGenAiClientOperationDurationModule: typeof import('./metrics.js').recordGenAiClientOperationDuration;
   let recordFlickerFrameModule: typeof import('./metrics.js').recordFlickerFrame;
+  let recordExitFailModule: typeof import('./metrics.js').recordExitFail;
   let recordAgentRunMetricsModule: typeof import('./metrics.js').recordAgentRunMetrics;
+  let recordLinesChangedModule: typeof import('./metrics.js').recordLinesChanged;
+  let recordSlowRenderModule: typeof import('./metrics.js').recordSlowRender;
 
   beforeEach(async () => {
     vi.resetModules();
@@ -133,7 +136,10 @@ describe('Telemetry Metrics', () => {
     recordGenAiClientOperationDurationModule =
       metricsJsModule.recordGenAiClientOperationDuration;
     recordFlickerFrameModule = metricsJsModule.recordFlickerFrame;
+    recordExitFailModule = metricsJsModule.recordExitFail;
     recordAgentRunMetricsModule = metricsJsModule.recordAgentRunMetrics;
+    recordLinesChangedModule = metricsJsModule.recordLinesChanged;
+    recordSlowRenderModule = metricsJsModule.recordSlowRender;
 
     const otelApiModule = await import('@opentelemetry/api');
 
@@ -170,6 +176,48 @@ describe('Telemetry Metrics', () => {
     });
   });
 
+  describe('recordExitFail', () => {
+    it('does not record metrics if not initialized', () => {
+      const config = makeFakeConfig({});
+      recordExitFailModule(config);
+      expect(mockCounterAddFn).not.toHaveBeenCalled();
+    });
+
+    it('records a exit fail event when initialized', () => {
+      const config = makeFakeConfig({});
+      initializeMetricsModule(config);
+      recordExitFailModule(config);
+
+      // Called for session, then for exit fail
+      expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
+      expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 1, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+      });
+    });
+  });
+
+  describe('recordSlowRender', () => {
+    it('does not record metrics if not initialized', () => {
+      const config = makeFakeConfig({});
+      recordSlowRenderModule(config, 123);
+      expect(mockHistogramRecordFn).not.toHaveBeenCalled();
+    });
+
+    it('records a slow render event when initialized', () => {
+      const config = makeFakeConfig({});
+      initializeMetricsModule(config);
+      recordSlowRenderModule(config, 123);
+
+      expect(mockHistogramRecordFn).toHaveBeenCalledWith(123, {
+        'session.id': 'test-session-id',
+        'installation.id': 'test-installation-id',
+        'user.email': 'test@example.com',
+      });
+    });
+  });
+
   describe('initializeMetrics', () => {
     const mockConfig = {
       getSessionId: () => 'test-session-id',
@@ -232,94 +280,78 @@ describe('Telemetry Metrics', () => {
       expect(mockCounterAddFn).not.toHaveBeenCalled();
     });
 
-    it('should record token usage with the correct attributes', () => {
+    it.each([
+      { type: 'input', tokens: 100, model: 'gemini-pro' },
+      { type: 'output', tokens: 50, model: 'gemini-pro' },
+      { type: 'thought', tokens: 25, model: 'gemini-pro' },
+      { type: 'cache', tokens: 75, model: 'gemini-pro' },
+      { type: 'tool', tokens: 125, model: 'gemini-pro' },
+      { type: 'input', tokens: 200, model: 'gemini-different-model' },
+    ])(
+      'should record token usage for $type type with $tokens tokens for model $model',
+      ({ type, tokens, model }) => {
+        initializeMetricsModule(mockConfig);
+        mockCounterAddFn.mockClear();
+
+        recordTokenUsageMetricsModule(mockConfig, tokens, {
+          model,
+          type: type as 'input' | 'output' | 'thought' | 'cache' | 'tool',
+        });
+
+        expect(mockCounterAddFn).toHaveBeenCalledWith(tokens, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          model,
+          type,
+        });
+      },
+    );
+  });
+
+  describe('recordLinesChanged metric', () => {
+    const mockConfig = {
+      getSessionId: () => 'test-session-id',
+      getTelemetryEnabled: () => true,
+    } as unknown as Config;
+
+    it('should not record lines added/removed if not initialized', () => {
+      recordLinesChangedModule(mockConfig, 10, 'added', {
+        function_name: 'fn',
+      });
+      recordLinesChangedModule(mockConfig, 5, 'removed', {
+        function_name: 'fn',
+      });
+      expect(mockCounterAddFn).not.toHaveBeenCalled();
+    });
+
+    it('should record lines added with function_name after initialization', () => {
       initializeMetricsModule(mockConfig);
-      recordTokenUsageMetricsModule(mockConfig, 100, {
-        model: 'gemini-pro',
-        type: 'input',
+      mockCounterAddFn.mockClear();
+      recordLinesChangedModule(mockConfig, 10, 'added', {
+        function_name: 'my-fn',
       });
-      expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
-      expect(mockCounterAddFn).toHaveBeenNthCalledWith(1, 1, {
+      expect(mockCounterAddFn).toHaveBeenCalledWith(10, {
         'session.id': 'test-session-id',
         'installation.id': 'test-installation-id',
         'user.email': 'test@example.com',
-      });
-      expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 100, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        model: 'gemini-pro',
-        type: 'input',
+        type: 'added',
+        function_name: 'my-fn',
       });
     });
 
-    it('should record token usage for different types', () => {
+    it('should record lines removed with function_name after initialization', () => {
       initializeMetricsModule(mockConfig);
       mockCounterAddFn.mockClear();
-
-      recordTokenUsageMetricsModule(mockConfig, 50, {
-        model: 'gemini-pro',
-        type: 'output',
+      recordLinesChangedModule(mockConfig, 7, 'removed', {
+        function_name: 'my-fn',
       });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(50, {
+      expect(mockCounterAddFn).toHaveBeenCalledWith(7, {
         'session.id': 'test-session-id',
         'installation.id': 'test-installation-id',
         'user.email': 'test@example.com',
-        model: 'gemini-pro',
-        type: 'output',
-      });
-
-      recordTokenUsageMetricsModule(mockConfig, 25, {
-        model: 'gemini-pro',
-        type: 'thought',
-      });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(25, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        model: 'gemini-pro',
-        type: 'thought',
-      });
-
-      recordTokenUsageMetricsModule(mockConfig, 75, {
-        model: 'gemini-pro',
-        type: 'cache',
-      });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(75, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        model: 'gemini-pro',
-        type: 'cache',
-      });
-
-      recordTokenUsageMetricsModule(mockConfig, 125, {
-        model: 'gemini-pro',
-        type: 'tool',
-      });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(125, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        model: 'gemini-pro',
-        type: 'tool',
-      });
-    });
-
-    it('should handle different models', () => {
-      initializeMetricsModule(mockConfig);
-      mockCounterAddFn.mockClear();
-
-      recordTokenUsageMetricsModule(mockConfig, 200, {
-        model: 'gemini-ultra',
-        type: 'input',
-      });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(200, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        model: 'gemini-ultra',
-        type: 'input',
+        type: 'removed',
+        function_name: 'my-fn',
       });
     });
   });
@@ -330,126 +362,81 @@ describe('Telemetry Metrics', () => {
       getTelemetryEnabled: () => true,
     } as unknown as Config;
 
-    it('should not record metrics if not initialized', () => {
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.CREATE,
-        lines: 10,
-        mimetype: 'text/plain',
-        extension: 'txt',
-      });
-      expect(mockCounterAddFn).not.toHaveBeenCalled();
-    });
+    type FileOperationTestCase = {
+      name: string;
+      initialized: boolean;
+      attributes: {
+        operation: FileOperation;
+        lines?: number;
+        mimetype?: string;
+        extension?: string;
+      };
+      shouldCall: boolean;
+    };
 
-    it('should record file creation with all attributes', () => {
-      initializeMetricsModule(mockConfig);
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.CREATE,
-        lines: 10,
-        mimetype: 'text/plain',
-        extension: 'txt',
-      });
+    it.each<FileOperationTestCase>([
+      {
+        name: 'should not record metrics if not initialized',
+        initialized: false,
+        attributes: {
+          operation: FileOperation.CREATE,
+          lines: 10,
+          mimetype: 'text/plain',
+          extension: 'txt',
+        },
+        shouldCall: false,
+      },
+      {
+        name: 'should record file creation with all attributes',
+        initialized: true,
+        attributes: {
+          operation: FileOperation.CREATE,
+          lines: 10,
+          mimetype: 'text/plain',
+          extension: 'txt',
+        },
+        shouldCall: true,
+      },
+      {
+        name: 'should record file read with minimal attributes',
+        initialized: true,
+        attributes: { operation: FileOperation.READ },
+        shouldCall: true,
+      },
+      {
+        name: 'should record file update with some attributes',
+        initialized: true,
+        attributes: {
+          operation: FileOperation.UPDATE,
+          mimetype: 'application/javascript',
+        },
+        shouldCall: true,
+      },
+      {
+        name: 'should record file update with no optional attributes',
+        initialized: true,
+        attributes: { operation: FileOperation.UPDATE },
+        shouldCall: true,
+      },
+    ])('$name', ({ initialized, attributes, shouldCall }) => {
+      if (initialized) {
+        initializeMetricsModule(mockConfig);
+        // The session start event also calls the counter.
+        mockCounterAddFn.mockClear();
+      }
 
-      expect(mockCounterAddFn).toHaveBeenCalledTimes(2);
-      expect(mockCounterAddFn).toHaveBeenNthCalledWith(1, 1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-      });
-      expect(mockCounterAddFn).toHaveBeenNthCalledWith(2, 1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        operation: FileOperation.CREATE,
-        lines: 10,
-        mimetype: 'text/plain',
-        extension: 'txt',
-      });
-    });
+      recordFileOperationMetricModule(mockConfig, attributes);
 
-    it('should record file read with minimal attributes', () => {
-      initializeMetricsModule(mockConfig);
-      mockCounterAddFn.mockClear();
-
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.READ,
-      });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        operation: FileOperation.READ,
-      });
-    });
-
-    it('should record file update with some attributes', () => {
-      initializeMetricsModule(mockConfig);
-      mockCounterAddFn.mockClear();
-
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.UPDATE,
-        mimetype: 'application/javascript',
-      });
-      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        operation: FileOperation.UPDATE,
-        mimetype: 'application/javascript',
-      });
-    });
-
-    it('should record file operation without diffStat', () => {
-      initializeMetricsModule(mockConfig);
-      mockCounterAddFn.mockClear();
-
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.UPDATE,
-      });
-
-      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        operation: FileOperation.UPDATE,
-      });
-    });
-
-    it('should record minimal file operation when optional parameters are undefined', () => {
-      initializeMetricsModule(mockConfig);
-      mockCounterAddFn.mockClear();
-
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.UPDATE,
-        lines: 10,
-        mimetype: 'text/plain',
-        extension: 'txt',
-      });
-
-      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        operation: FileOperation.UPDATE,
-        lines: 10,
-        mimetype: 'text/plain',
-        extension: 'txt',
-      });
-    });
-
-    it('should not include diffStat attributes when diffStat is not provided', () => {
-      initializeMetricsModule(mockConfig);
-      mockCounterAddFn.mockClear();
-
-      recordFileOperationMetricModule(mockConfig, {
-        operation: FileOperation.UPDATE,
-      });
-
-      expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
-        'session.id': 'test-session-id',
-        'installation.id': 'test-installation-id',
-        'user.email': 'test@example.com',
-        operation: FileOperation.UPDATE,
-      });
+      if (shouldCall) {
+        expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          ...attributes,
+        });
+      } else {
+        expect(mockCounterAddFn).not.toHaveBeenCalled();
+      }
     });
   });
 
@@ -871,80 +858,60 @@ describe('Telemetry Metrics', () => {
     });
 
     describe('recordMemoryUsage', () => {
-      it('should record memory usage for different memory types', () => {
-        initializeMetricsModule(mockConfig);
-        mockHistogramRecordFn.mockClear();
-
-        recordMemoryUsageModule(mockConfig, 15728640, {
+      it.each([
+        {
           memory_type: MemoryMetricType.HEAP_USED,
           component: 'startup',
-        });
-
-        expect(mockHistogramRecordFn).toHaveBeenCalledWith(15728640, {
-          'session.id': 'test-session-id',
-          'installation.id': 'test-installation-id',
-          'user.email': 'test@example.com',
-          memory_type: 'heap_used',
-          component: 'startup',
-        });
-      });
-
-      it('should record memory usage for all memory metric types', () => {
-        initializeMetricsModule(mockConfig);
-        mockHistogramRecordFn.mockClear();
-
-        recordMemoryUsageModule(mockConfig, 31457280, {
+          value: 15728640,
+        },
+        {
           memory_type: MemoryMetricType.HEAP_TOTAL,
           component: 'api_call',
-        });
-        recordMemoryUsageModule(mockConfig, 2097152, {
+          value: 31457280,
+        },
+        {
           memory_type: MemoryMetricType.EXTERNAL,
           component: 'tool_execution',
-        });
-        recordMemoryUsageModule(mockConfig, 41943040, {
+          value: 2097152,
+        },
+        {
           memory_type: MemoryMetricType.RSS,
           component: 'memory_monitor',
-        });
-
-        expect(mockHistogramRecordFn).toHaveBeenCalledTimes(3); // One for each call
-        expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(1, 31457280, {
-          'session.id': 'test-session-id',
-          'installation.id': 'test-installation-id',
-          'user.email': 'test@example.com',
-          memory_type: 'heap_total',
-          component: 'api_call',
-        });
-        expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(2, 2097152, {
-          'session.id': 'test-session-id',
-          'installation.id': 'test-installation-id',
-          'user.email': 'test@example.com',
-          memory_type: 'external',
-          component: 'tool_execution',
-        });
-        expect(mockHistogramRecordFn).toHaveBeenNthCalledWith(3, 41943040, {
-          'session.id': 'test-session-id',
-          'installation.id': 'test-installation-id',
-          'user.email': 'test@example.com',
-          memory_type: 'rss',
-          component: 'memory_monitor',
-        });
-      });
-
-      it('should record memory usage without component', () => {
-        initializeMetricsModule(mockConfig);
-        mockHistogramRecordFn.mockClear();
-
-        recordMemoryUsageModule(mockConfig, 15728640, {
+          value: 41943040,
+        },
+        {
           memory_type: MemoryMetricType.HEAP_USED,
-        });
+          component: undefined,
+          value: 15728640,
+        },
+      ])(
+        'should record memory usage for $memory_type',
+        ({ memory_type, component, value }) => {
+          initializeMetricsModule(mockConfig);
+          mockHistogramRecordFn.mockClear();
 
-        expect(mockHistogramRecordFn).toHaveBeenCalledWith(15728640, {
-          'session.id': 'test-session-id',
-          'installation.id': 'test-installation-id',
-          'user.email': 'test@example.com',
-          memory_type: 'heap_used',
-        });
-      });
+          recordMemoryUsageModule(mockConfig, value, {
+            memory_type,
+            component,
+          });
+
+          const expectedAttributes: Record<string, unknown> = {
+            'session.id': 'test-session-id',
+            'installation.id': 'test-installation-id',
+            'user.email': 'test@example.com',
+            memory_type,
+          };
+
+          if (component) {
+            expectedAttributes['component'] = component;
+          }
+
+          expect(mockHistogramRecordFn).toHaveBeenCalledWith(
+            value,
+            expectedAttributes,
+          );
+        },
+      );
     });
 
     describe('recordCpuUsage', () => {
@@ -1375,5 +1342,115 @@ describe('Telemetry Metrics', () => {
         expect(mockHistogramRecordFn).not.toHaveBeenCalled();
       });
     });
+
+    describe('recordHookCallMetrics', () => {
+      let recordHookCallMetricsModule: typeof import('./metrics.js').recordHookCallMetrics;
+
+      beforeEach(async () => {
+        recordHookCallMetricsModule = (await import('./metrics.js'))
+          .recordHookCallMetrics;
+      });
+
+      it('should record hook call metrics with counter and histogram', () => {
+        initializeMetricsModule(mockConfig);
+        mockCounterAddFn.mockClear();
+        mockHistogramRecordFn.mockClear();
+
+        recordHookCallMetricsModule(
+          mockConfig,
+          'BeforeTool',
+          'test-hook',
+          150,
+          true,
+        );
+
+        // Verify counter recorded
+        expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          hook_event_name: 'BeforeTool',
+          hook_name: 'test-hook',
+          success: true,
+        });
+
+        // Verify histogram recorded
+        expect(mockHistogramRecordFn).toHaveBeenCalledWith(150, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          hook_event_name: 'BeforeTool',
+          hook_name: 'test-hook',
+          success: true,
+        });
+      });
+
+      it('should always sanitize hook names regardless of content', () => {
+        initializeMetricsModule(mockConfig);
+        mockCounterAddFn.mockClear();
+
+        // Test with a command that has sensitive information
+        recordHookCallMetricsModule(
+          mockConfig,
+          'BeforeTool',
+          '/path/to/.gemini/hooks/check-secrets.sh --api-key=abc123',
+          150,
+          true,
+        );
+
+        // Verify hook name is sanitized (detailed sanitization tested in hook-call-event.test.ts)
+        expect(mockCounterAddFn).toHaveBeenCalledWith(1, {
+          'session.id': 'test-session-id',
+          'installation.id': 'test-installation-id',
+          'user.email': 'test@example.com',
+          hook_event_name: 'BeforeTool',
+          hook_name: 'check-secrets.sh', // Sanitized
+          success: true,
+        });
+      });
+
+      it('should track both success and failure', () => {
+        initializeMetricsModule(mockConfig);
+        mockCounterAddFn.mockClear();
+
+        // Success case
+        recordHookCallMetricsModule(
+          mockConfig,
+          'BeforeTool',
+          'test-hook',
+          100,
+          true,
+        );
+
+        expect(mockCounterAddFn).toHaveBeenNthCalledWith(
+          1,
+          1,
+          expect.objectContaining({
+            hook_event_name: 'BeforeTool',
+            hook_name: 'test-hook',
+            success: true,
+          }),
+        );
+
+        // Failure case
+        recordHookCallMetricsModule(
+          mockConfig,
+          'AfterTool',
+          'test-hook',
+          150,
+          false,
+        );
+
+        expect(mockCounterAddFn).toHaveBeenNthCalledWith(
+          2,
+          1,
+          expect.objectContaining({
+            hook_event_name: 'AfterTool',
+            hook_name: 'test-hook',
+            success: false,
+          }),
+        );
+      });
+    });
   });
 });
