@@ -17,7 +17,7 @@ import type { RipGrepToolParams } from './ripGrep.js';
 import { canUseRipgrep, RipGrepTool, ensureRgPath } from './ripGrep.js';
 import path from 'node:path';
 import fs from 'node:fs/promises';
-import os, { EOL } from 'node:os';
+import os from 'node:os';
 import type { Config } from '../config/config.js';
 import { Storage } from '../config/storage.js';
 import { createMockWorkspaceContext } from '../test-utils/mockWorkspaceContext.js';
@@ -39,6 +39,10 @@ const downloadRipGrepMock = vi.mocked(downloadRipGrep);
 const originalGetGlobalBinDir = Storage.getGlobalBinDir.bind(Storage);
 const storageSpy = vi.spyOn(Storage, 'getGlobalBinDir');
 
+function getRipgrepBinaryName() {
+  return process.platform === 'win32' ? 'rg.exe' : 'rg';
+}
+
 describe('canUseRipgrep', () => {
   let tempRootDir: string;
   let binDir: string;
@@ -58,9 +62,7 @@ describe('canUseRipgrep', () => {
   });
 
   it('should return true if ripgrep already exists', async () => {
-    const candidateNames =
-      process.platform === 'win32' ? ['rg.exe', 'rg'] : ['rg'];
-    const existingPath = path.join(binDir, candidateNames[0]);
+    const existingPath = path.join(binDir, getRipgrepBinaryName());
     await fs.writeFile(existingPath, '');
 
     const result = await canUseRipgrep();
@@ -69,9 +71,7 @@ describe('canUseRipgrep', () => {
   });
 
   it('should download ripgrep and return true if it does not exist initially', async () => {
-    const candidateNames =
-      process.platform === 'win32' ? ['rg.exe', 'rg'] : ['rg'];
-    const expectedPath = path.join(binDir, candidateNames[0]);
+    const expectedPath = path.join(binDir, getRipgrepBinaryName());
 
     downloadRipGrepMock.mockImplementation(async () => {
       await fs.writeFile(expectedPath, '');
@@ -100,9 +100,7 @@ describe('canUseRipgrep', () => {
   });
 
   it('should only download once when called concurrently', async () => {
-    const candidateNames =
-      process.platform === 'win32' ? ['rg.exe', 'rg'] : ['rg'];
-    const expectedPath = path.join(binDir, candidateNames[0]);
+    const expectedPath = path.join(binDir, getRipgrepBinaryName());
 
     downloadRipGrepMock.mockImplementation(
       () =>
@@ -146,9 +144,7 @@ describe('ensureRgPath', () => {
   });
 
   it('should return rg path if ripgrep already exists', async () => {
-    const candidateNames =
-      process.platform === 'win32' ? ['rg.exe', 'rg'] : ['rg'];
-    const existingPath = path.join(binDir, candidateNames[0]);
+    const existingPath = path.join(binDir, getRipgrepBinaryName());
     await fs.writeFile(existingPath, '');
 
     const rgPath = await ensureRgPath();
@@ -157,9 +153,7 @@ describe('ensureRgPath', () => {
   });
 
   it('should return rg path if ripgrep is downloaded successfully', async () => {
-    const candidateNames =
-      process.platform === 'win32' ? ['rg.exe', 'rg'] : ['rg'];
-    const expectedPath = path.join(binDir, candidateNames[0]);
+    const expectedPath = path.join(binDir, getRipgrepBinaryName());
 
     downloadRipGrepMock.mockImplementation(async () => {
       await fs.writeFile(expectedPath, '');
@@ -301,55 +295,60 @@ describe('RipGrepTool', () => {
   });
 
   describe('validateToolParams', () => {
-    it('should return null for valid params (pattern only)', () => {
-      const params: RipGrepToolParams = { pattern: 'hello' };
-      expect(grepTool.validateToolParams(params)).toBeNull();
-    });
-
-    it('should return null for valid params (pattern and path)', () => {
-      const params: RipGrepToolParams = { pattern: 'hello', path: '.' };
-      expect(grepTool.validateToolParams(params)).toBeNull();
-    });
-
-    it('should return null for valid params (pattern, path, and include)', () => {
-      const params: RipGrepToolParams = {
-        pattern: 'hello',
-        path: '.',
-        include: '*.txt',
-      };
-      expect(grepTool.validateToolParams(params)).toBeNull();
-    });
+    it.each([
+      {
+        name: 'pattern only',
+        params: { pattern: 'hello' },
+        expected: null,
+      },
+      {
+        name: 'pattern and path',
+        params: { pattern: 'hello', dir_path: '.' },
+        expected: null,
+      },
+      {
+        name: 'pattern, path, and include',
+        params: { pattern: 'hello', dir_path: '.', include: '*.txt' },
+        expected: null,
+      },
+      {
+        name: 'invalid regex pattern',
+        params: { pattern: '[[' },
+        expected: null,
+      },
+    ])(
+      'should return null for valid params ($name)',
+      ({ params, expected }) => {
+        expect(grepTool.validateToolParams(params)).toBe(expected);
+      },
+    );
 
     it('should return error if pattern is missing', () => {
-      const params = { path: '.' } as unknown as RipGrepToolParams;
+      const params = { dir_path: '.' } as unknown as RipGrepToolParams;
       expect(grepTool.validateToolParams(params)).toBe(
         `params must have required property 'pattern'`,
       );
     });
 
-    it('should return null for what would be an invalid regex pattern', () => {
-      const params: RipGrepToolParams = { pattern: '[[' };
-      expect(grepTool.validateToolParams(params)).toBeNull();
-    });
-
     it('should return error if path does not exist', () => {
       const params: RipGrepToolParams = {
         pattern: 'hello',
-        path: 'nonexistent',
+        dir_path: 'nonexistent',
       };
       // Check for the core error message, as the full path might vary
       expect(grepTool.validateToolParams(params)).toContain(
-        'Failed to access path stats for',
+        'Path does not exist',
       );
       expect(grepTool.validateToolParams(params)).toContain('nonexistent');
     });
 
-    it('should return error if path is a file, not a directory', async () => {
+    it('should allow path to be a file', async () => {
       const filePath = path.join(tempRootDir, 'fileA.txt');
-      const params: RipGrepToolParams = { pattern: 'hello', path: filePath };
-      expect(grepTool.validateToolParams(params)).toContain(
-        `Path is not a directory: ${filePath}`,
-      );
+      const params: RipGrepToolParams = {
+        pattern: 'hello',
+        dir_path: filePath,
+      };
+      expect(grepTool.validateToolParams(params)).toBeNull();
     });
   });
 
@@ -357,7 +356,34 @@ describe('RipGrepTool', () => {
     it('should find matches for a simple pattern in all files', async () => {
       mockSpawn.mockImplementationOnce(
         createMockSpawn({
-          outputData: `fileA.txt:1:hello world${EOL}fileA.txt:2:second line with world${EOL}sub/fileC.txt:1:another world in sub dir${EOL}`,
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileA.txt' },
+                line_number: 1,
+                lines: { text: 'hello world\n' },
+              },
+            }) +
+            '\n' +
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileA.txt' },
+                line_number: 2,
+                lines: { text: 'second line with world\n' },
+              },
+            }) +
+            '\n' +
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'sub/fileC.txt' },
+                line_number: 1,
+                lines: { text: 'another world in sub dir\n' },
+              },
+            }) +
+            '\n',
           exitCode: 0,
         }),
       );
@@ -366,7 +392,7 @@ describe('RipGrepTool', () => {
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
       expect(result.llmContent).toContain(
-        'Found 3 matches for pattern "world" in the workspace directory',
+        'Found 3 matches for pattern "world" in path "."',
       );
       expect(result.llmContent).toContain('File: fileA.txt');
       expect(result.llmContent).toContain('L1: hello world');
@@ -382,12 +408,20 @@ describe('RipGrepTool', () => {
       // Setup specific mock for this test - searching in 'sub' should only return matches from that directory
       mockSpawn.mockImplementationOnce(
         createMockSpawn({
-          outputData: `fileC.txt:1:another world in sub dir${EOL}`,
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileC.txt' },
+                line_number: 1,
+                lines: { text: 'another world in sub dir\n' },
+              },
+            }) + '\n',
           exitCode: 0,
         }),
       );
 
-      const params: RipGrepToolParams = { pattern: 'world', path: 'sub' };
+      const params: RipGrepToolParams = { pattern: 'world', dir_path: 'sub' };
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
       expect(result.llmContent).toContain(
@@ -402,7 +436,15 @@ describe('RipGrepTool', () => {
       // Setup specific mock for this test
       mockSpawn.mockImplementationOnce(
         createMockSpawn({
-          outputData: `fileB.js:2:function baz() { return "hello"; }${EOL}`,
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileB.js' },
+                line_number: 2,
+                lines: { text: 'function baz() { return "hello"; }\n' },
+              },
+            }) + '\n',
           exitCode: 0,
         }),
       );
@@ -411,7 +453,7 @@ describe('RipGrepTool', () => {
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
       expect(result.llmContent).toContain(
-        'Found 1 match for pattern "hello" in the workspace directory (filter: "*.js"):',
+        'Found 1 match for pattern "hello" in path "." (filter: "*.js"):',
       );
       expect(result.llmContent).toContain('File: fileB.js');
       expect(result.llmContent).toContain(
@@ -452,7 +494,18 @@ describe('RipGrepTool', () => {
 
           if (onData) {
             // Only return match from the .js file in sub directory
-            onData(Buffer.from(`another.js:1:const greeting = "hello";${EOL}`));
+            onData(
+              Buffer.from(
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'another.js' },
+                    line_number: 1,
+                    lines: { text: 'const greeting = "hello";\n' },
+                  },
+                }) + '\n',
+              ),
+            );
           }
           if (onClose) {
             onClose(0);
@@ -464,7 +517,7 @@ describe('RipGrepTool', () => {
 
       const params: RipGrepToolParams = {
         pattern: 'hello',
-        path: 'sub',
+        dir_path: 'sub',
         include: '*.js',
       };
       const invocation = grepTool.build(params);
@@ -489,7 +542,7 @@ describe('RipGrepTool', () => {
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
       expect(result.llmContent).toContain(
-        'No matches found for pattern "nonexistentpattern" in the workspace directory.',
+        'No matches found for pattern "nonexistentpattern" in path ".".',
       );
       expect(result.returnDisplay).toBe('No matches found');
     });
@@ -537,7 +590,18 @@ describe('RipGrepTool', () => {
 
           if (onData) {
             // Return match for the regex pattern
-            onData(Buffer.from(`fileB.js:1:const foo = "bar";${EOL}`));
+            onData(
+              Buffer.from(
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'fileB.js' },
+                    line_number: 1,
+                    lines: { text: 'const foo = "bar";\n' },
+                  },
+                }) + '\n',
+              ),
+            );
           }
           if (onClose) {
             onClose(0);
@@ -551,7 +615,7 @@ describe('RipGrepTool', () => {
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
       expect(result.llmContent).toContain(
-        'Found 1 match for pattern "foo.*bar" in the workspace directory:',
+        'Found 1 match for pattern "foo.*bar" in path ".":',
       );
       expect(result.llmContent).toContain('File: fileB.js');
       expect(result.llmContent).toContain('L1: const foo = "bar";');
@@ -586,7 +650,24 @@ describe('RipGrepTool', () => {
             // Return case-insensitive matches for 'HELLO'
             onData(
               Buffer.from(
-                `fileA.txt:1:hello world${EOL}fileB.js:2:function baz() { return "hello"; }${EOL}`,
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'fileA.txt' },
+                    line_number: 1,
+                    lines: { text: 'hello world\n' },
+                  },
+                }) +
+                  '\n' +
+                  JSON.stringify({
+                    type: 'match',
+                    data: {
+                      path: { text: 'fileB.js' },
+                      line_number: 2,
+                      lines: { text: 'function baz() { return "hello"; }\n' },
+                    },
+                  }) +
+                  '\n',
               ),
             );
           }
@@ -602,7 +683,7 @@ describe('RipGrepTool', () => {
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
       expect(result.llmContent).toContain(
-        'Found 2 matches for pattern "HELLO" in the workspace directory:',
+        'Found 2 matches for pattern "HELLO" in path ".":',
       );
       expect(result.llmContent).toContain('File: fileA.txt');
       expect(result.llmContent).toContain('L1: hello world');
@@ -613,7 +694,7 @@ describe('RipGrepTool', () => {
     });
 
     it('should throw an error if params are invalid', async () => {
-      const params = { path: '.' } as unknown as RipGrepToolParams; // Invalid: pattern missing
+      const params = { dir_path: '.' } as unknown as RipGrepToolParams; // Invalid: pattern missing
       expect(() => grepTool.build(params)).toThrow(
         /params must have required property 'pattern'/,
       );
@@ -634,7 +715,7 @@ describe('RipGrepTool', () => {
   });
 
   describe('multi-directory workspace', () => {
-    it('should search across all workspace directories when no path is specified', async () => {
+    it('should search only CWD when no path is specified (default behavior)', async () => {
       // Create additional directory with test files
       const secondDir = await fs.mkdtemp(
         path.join(os.tmpdir(), 'grep-tool-second-'),
@@ -688,18 +769,54 @@ describe('RipGrepTool', () => {
           if (callCount === 1) {
             // First directory (tempRootDir)
             outputData =
-              [
-                'fileA.txt:1:hello world',
-                'fileA.txt:2:second line with world',
-                'sub/fileC.txt:1:another world in sub dir',
-              ].join(EOL) + EOL;
+              JSON.stringify({
+                type: 'match',
+                data: {
+                  path: { text: 'fileA.txt' },
+                  line_number: 1,
+                  lines: { text: 'hello world\n' },
+                },
+              }) +
+              '\n' +
+              JSON.stringify({
+                type: 'match',
+                data: {
+                  path: { text: 'fileA.txt' },
+                  line_number: 2,
+                  lines: { text: 'second line with world\n' },
+                },
+              }) +
+              '\n' +
+              JSON.stringify({
+                type: 'match',
+                data: {
+                  path: { text: 'sub/fileC.txt' },
+                  line_number: 1,
+                  lines: { text: 'another world in sub dir\n' },
+                },
+              }) +
+              '\n';
           } else if (callCount === 2) {
             // Second directory (secondDir)
             outputData =
-              [
-                'other.txt:2:world in second',
-                'another.js:1:function world() { return "test"; }',
-              ].join(EOL) + EOL;
+              JSON.stringify({
+                type: 'match',
+                data: {
+                  path: { text: 'other.txt' },
+                  line_number: 2,
+                  lines: { text: 'world in second\n' },
+                },
+              }) +
+              '\n' +
+              JSON.stringify({
+                type: 'match',
+                data: {
+                  path: { text: 'another.js' },
+                  line_number: 1,
+                  lines: { text: 'function world() { return "test"; }\n' },
+                },
+              }) +
+              '\n';
           }
 
           if (stdoutDataHandler && outputData) {
@@ -719,9 +836,9 @@ describe('RipGrepTool', () => {
       const invocation = multiDirGrepTool.build(params);
       const result = await invocation.execute(abortSignal);
 
-      // Should find matches in both directories
+      // Should find matches in CWD only (default behavior now)
       expect(result.llmContent).toContain(
-        'Found 5 matches for pattern "world"',
+        'Found 3 matches for pattern "world" in path "."',
       );
 
       // Matches from first directory
@@ -731,11 +848,11 @@ describe('RipGrepTool', () => {
       expect(result.llmContent).toContain('fileC.txt');
       expect(result.llmContent).toContain('L1: another world in sub dir');
 
-      // Matches from both directories
-      expect(result.llmContent).toContain('other.txt');
-      expect(result.llmContent).toContain('L2: world in second');
-      expect(result.llmContent).toContain('another.js');
-      expect(result.llmContent).toContain('L1: function world()');
+      // Should NOT find matches from second directory
+      expect(result.llmContent).not.toContain('other.txt');
+      expect(result.llmContent).not.toContain('world in second');
+      expect(result.llmContent).not.toContain('another.js');
+      expect(result.llmContent).not.toContain('function world()');
 
       // Clean up
       await fs.rm(secondDir, { recursive: true, force: true });
@@ -786,7 +903,18 @@ describe('RipGrepTool', () => {
           )?.[1];
 
           if (onData) {
-            onData(Buffer.from(`fileC.txt:1:another world in sub dir${EOL}`));
+            onData(
+              Buffer.from(
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'fileC.txt' },
+                    line_number: 1,
+                    lines: { text: 'another world in sub dir\n' },
+                  },
+                }) + '\n',
+              ),
+            );
           }
           if (onClose) {
             onClose(0);
@@ -799,7 +927,7 @@ describe('RipGrepTool', () => {
       const multiDirGrepTool = new RipGrepTool(multiDirConfig);
 
       // Search only in the 'sub' directory of the first workspace
-      const params: RipGrepToolParams = { pattern: 'world', path: 'sub' };
+      const params: RipGrepToolParams = { pattern: 'world', dir_path: 'sub' };
       const invocation = multiDirGrepTool.build(params);
       const result = await invocation.execute(abortSignal);
 
@@ -881,84 +1009,33 @@ describe('RipGrepTool', () => {
 
   describe('error handling and edge cases', () => {
     it('should handle workspace boundary violations', () => {
-      const params: RipGrepToolParams = { pattern: 'test', path: '../outside' };
+      const params: RipGrepToolParams = {
+        pattern: 'test',
+        dir_path: '../outside',
+      };
       expect(() => grepTool.build(params)).toThrow(/Path validation failed/);
     });
 
-    it('should handle empty directories gracefully', async () => {
-      const emptyDir = path.join(tempRootDir, 'empty');
-      await fs.mkdir(emptyDir);
+    it.each([
+      {
+        name: 'empty directories',
+        setup: async () => {
+          const emptyDir = path.join(tempRootDir, 'empty');
+          await fs.mkdir(emptyDir);
+          return { pattern: 'test', dir_path: 'empty' };
+        },
+      },
+      {
+        name: 'empty files',
+        setup: async () => {
+          await fs.writeFile(path.join(tempRootDir, 'empty.txt'), '');
+          return { pattern: 'anything' };
+        },
+      },
+    ])('should handle $name gracefully', async ({ setup }) => {
+      mockSpawn.mockImplementationOnce(createMockSpawn({ exitCode: 1 }));
 
-      // Setup specific mock for this test - searching in empty directory should return no matches
-      mockSpawn.mockImplementationOnce(() => {
-        const mockProcess = {
-          stdout: {
-            on: vi.fn(),
-            removeListener: vi.fn(),
-          },
-          stderr: {
-            on: vi.fn(),
-            removeListener: vi.fn(),
-          },
-          on: vi.fn(),
-          removeListener: vi.fn(),
-          kill: vi.fn(),
-        };
-
-        setTimeout(() => {
-          const onClose = mockProcess.on.mock.calls.find(
-            (call) => call[0] === 'close',
-          )?.[1];
-
-          if (onClose) {
-            onClose(1);
-          }
-        }, 0);
-
-        return mockProcess as unknown as ChildProcess;
-      });
-
-      const params: RipGrepToolParams = { pattern: 'test', path: 'empty' };
-      const invocation = grepTool.build(params);
-      const result = await invocation.execute(abortSignal);
-
-      expect(result.llmContent).toContain('No matches found');
-      expect(result.returnDisplay).toBe('No matches found');
-    });
-
-    it('should handle empty files correctly', async () => {
-      await fs.writeFile(path.join(tempRootDir, 'empty.txt'), '');
-
-      // Setup specific mock for this test - searching for anything in empty files should return no matches
-      mockSpawn.mockImplementationOnce(() => {
-        const mockProcess = {
-          stdout: {
-            on: vi.fn(),
-            removeListener: vi.fn(),
-          },
-          stderr: {
-            on: vi.fn(),
-            removeListener: vi.fn(),
-          },
-          on: vi.fn(),
-          removeListener: vi.fn(),
-          kill: vi.fn(),
-        };
-
-        setTimeout(() => {
-          const onClose = mockProcess.on.mock.calls.find(
-            (call) => call[0] === 'close',
-          )?.[1];
-
-          if (onClose) {
-            onClose(1);
-          }
-        }, 0);
-
-        return mockProcess as unknown as ChildProcess;
-      });
-
-      const params: RipGrepToolParams = { pattern: 'anything' };
+      const params = await setup();
       const invocation = grepTool.build(params);
       const result = await invocation.execute(abortSignal);
 
@@ -999,7 +1076,14 @@ describe('RipGrepTool', () => {
           if (onData) {
             onData(
               Buffer.from(
-                `${specialFileName}:1:hello world with special chars${EOL}`,
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: specialFileName },
+                    line_number: 1,
+                    lines: { text: 'hello world with special chars\n' },
+                  },
+                }) + '\n',
               ),
             );
           }
@@ -1054,7 +1138,14 @@ describe('RipGrepTool', () => {
           if (onData) {
             onData(
               Buffer.from(
-                `a/b/c/d/e/deep.txt:1:content in deep directory${EOL}`,
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'a/b/c/d/e/deep.txt' },
+                    line_number: 1,
+                    lines: { text: 'content in deep directory\n' },
+                  },
+                }) + '\n',
               ),
             );
           }
@@ -1109,7 +1200,14 @@ describe('RipGrepTool', () => {
           if (onData) {
             onData(
               Buffer.from(
-                `code.js:1:function getName() { return "test"; }${EOL}`,
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'code.js' },
+                    line_number: 1,
+                    lines: { text: 'function getName() { return "test"; }\n' },
+                  },
+                }) + '\n',
               ),
             );
           }
@@ -1162,7 +1260,33 @@ describe('RipGrepTool', () => {
           if (onData) {
             onData(
               Buffer.from(
-                `case.txt:1:Hello World${EOL}case.txt:2:hello world${EOL}case.txt:3:HELLO WORLD${EOL}`,
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'case.txt' },
+                    line_number: 1,
+                    lines: { text: 'Hello World\n' },
+                  },
+                }) +
+                  '\n' +
+                  JSON.stringify({
+                    type: 'match',
+                    data: {
+                      path: { text: 'case.txt' },
+                      line_number: 2,
+                      lines: { text: 'hello world\n' },
+                    },
+                  }) +
+                  '\n' +
+                  JSON.stringify({
+                    type: 'match',
+                    data: {
+                      path: { text: 'case.txt' },
+                      line_number: 3,
+                      lines: { text: 'HELLO WORLD\n' },
+                    },
+                  }) +
+                  '\n',
               ),
             );
           }
@@ -1214,7 +1338,18 @@ describe('RipGrepTool', () => {
           )?.[1];
 
           if (onData) {
-            onData(Buffer.from(`special.txt:1:Price: $19.99${EOL}`));
+            onData(
+              Buffer.from(
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'special.txt' },
+                    line_number: 1,
+                    lines: { text: 'Price: $19.99\n' },
+                  },
+                }) + '\n',
+              ),
+            );
           }
           if (onClose) {
             onClose(0);
@@ -1273,7 +1408,24 @@ describe('RipGrepTool', () => {
           if (onData) {
             onData(
               Buffer.from(
-                `test.ts:1:typescript content${EOL}test.tsx:1:tsx content${EOL}`,
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'test.ts' },
+                    line_number: 1,
+                    lines: { text: 'typescript content\n' },
+                  },
+                }) +
+                  '\n' +
+                  JSON.stringify({
+                    type: 'match',
+                    data: {
+                      path: { text: 'test.tsx' },
+                      line_number: 1,
+                      lines: { text: 'tsx content\n' },
+                    },
+                  }) +
+                  '\n',
               ),
             );
           }
@@ -1331,7 +1483,18 @@ describe('RipGrepTool', () => {
           )?.[1];
 
           if (onData) {
-            onData(Buffer.from(`src/main.ts:1:source code${EOL}`));
+            onData(
+              Buffer.from(
+                JSON.stringify({
+                  type: 'match',
+                  data: {
+                    path: { text: 'src/main.ts' },
+                    line_number: 1,
+                    lines: { text: 'source code\n' },
+                  },
+                }) + '\n',
+              ),
+            );
           }
           if (onClose) {
             onClose(0);
@@ -1353,37 +1516,222 @@ describe('RipGrepTool', () => {
     });
   });
 
-  describe('getDescription', () => {
-    it('should generate correct description with pattern only', () => {
-      const params: RipGrepToolParams = { pattern: 'testPattern' };
-      const invocation = grepTool.build(params);
-      expect(invocation.getDescription()).toBe("'testPattern'");
+  describe('advanced search options', () => {
+    it('should handle case_sensitive parameter', async () => {
+      // Case-insensitive search (default)
+      mockSpawn.mockImplementationOnce(
+        createMockSpawn({
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileA.txt' },
+                line_number: 1,
+                lines: { text: 'hello world\n' },
+              },
+            }) + '\n',
+          exitCode: 0,
+        }),
+      );
+      let params: RipGrepToolParams = { pattern: 'HELLO' };
+      let invocation = grepTool.build(params);
+      let result = await invocation.execute(abortSignal);
+      expect(mockSpawn).toHaveBeenLastCalledWith(
+        expect.anything(),
+        expect.arrayContaining(['--ignore-case']),
+        expect.anything(),
+      );
+      expect(result.llmContent).toContain('Found 1 match for pattern "HELLO"');
+      expect(result.llmContent).toContain('L1: hello world');
+
+      // Case-sensitive search
+      mockSpawn.mockImplementationOnce(
+        createMockSpawn({
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileA.txt' },
+                line_number: 1,
+                lines: { text: 'HELLO world\n' },
+              },
+            }) + '\n',
+          exitCode: 0,
+        }),
+      );
+      params = { pattern: 'HELLO', case_sensitive: true };
+      invocation = grepTool.build(params);
+      result = await invocation.execute(abortSignal);
+      expect(mockSpawn).toHaveBeenLastCalledWith(
+        expect.anything(),
+        expect.not.arrayContaining(['--ignore-case']),
+        expect.anything(),
+      );
+      expect(result.llmContent).toContain('Found 1 match for pattern "HELLO"');
+      expect(result.llmContent).toContain('L1: HELLO world');
     });
 
-    it('should generate correct description with pattern and include', () => {
+    it.each([
+      {
+        name: 'fixed_strings parameter',
+        params: { pattern: 'hello.world', fixed_strings: true },
+        mockOutput: {
+          path: { text: 'fileA.txt' },
+          line_number: 1,
+          lines: { text: 'hello.world\n' },
+        },
+        expectedArgs: ['--fixed-strings'],
+        expectedPattern: 'hello.world',
+      },
+    ])(
+      'should handle $name',
+      async ({ params, mockOutput, expectedArgs, expectedPattern }) => {
+        mockSpawn.mockImplementationOnce(
+          createMockSpawn({
+            outputData:
+              JSON.stringify({ type: 'match', data: mockOutput }) + '\n',
+            exitCode: 0,
+          }),
+        );
+
+        const invocation = grepTool.build(params);
+        const result = await invocation.execute(abortSignal);
+
+        expect(mockSpawn).toHaveBeenLastCalledWith(
+          expect.anything(),
+          expect.arrayContaining(expectedArgs),
+          expect.anything(),
+        );
+        expect(result.llmContent).toContain(
+          `Found 1 match for pattern "${expectedPattern}"`,
+        );
+      },
+    );
+
+    it('should handle no_ignore parameter', async () => {
+      mockSpawn.mockImplementationOnce(
+        createMockSpawn({
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'ignored.log' },
+                line_number: 1,
+                lines: { text: 'secret log entry\n' },
+              },
+            }) + '\n',
+          exitCode: 0,
+        }),
+      );
+
+      const params: RipGrepToolParams = { pattern: 'secret', no_ignore: true };
+      const invocation = grepTool.build(params);
+      const result = await invocation.execute(abortSignal);
+
+      expect(mockSpawn).toHaveBeenLastCalledWith(
+        expect.anything(),
+        expect.arrayContaining(['--no-ignore']),
+        expect.anything(),
+      );
+
+      expect(mockSpawn).toHaveBeenLastCalledWith(
+        expect.anything(),
+        expect.not.arrayContaining(['--glob', '!node_modules']),
+        expect.anything(),
+      );
+      expect(result.llmContent).toContain('Found 1 match for pattern "secret"');
+      expect(result.llmContent).toContain('File: ignored.log');
+      expect(result.llmContent).toContain('L1: secret log entry');
+    });
+
+    it('should handle context parameters', async () => {
+      mockSpawn.mockImplementationOnce(
+        createMockSpawn({
+          outputData:
+            JSON.stringify({
+              type: 'match',
+              data: {
+                path: { text: 'fileA.txt' },
+                line_number: 2,
+                lines: { text: 'second line with world\n' },
+                lines_before: [{ text: 'hello world\n' }],
+                lines_after: [
+                  { text: 'third line\n' },
+                  { text: 'fourth line\n' },
+                ],
+              },
+            }) + '\n',
+          exitCode: 0,
+        }),
+      );
+
       const params: RipGrepToolParams = {
-        pattern: 'testPattern',
-        include: '*.ts',
+        pattern: 'world',
+        context: 1,
+        after: 2,
+        before: 1,
       };
       const invocation = grepTool.build(params);
-      expect(invocation.getDescription()).toBe("'testPattern' in *.ts");
+      const result = await invocation.execute(abortSignal);
+
+      expect(mockSpawn).toHaveBeenLastCalledWith(
+        expect.anything(),
+        expect.arrayContaining([
+          '--context',
+          '1',
+          '--after-context',
+          '2',
+          '--before-context',
+          '1',
+        ]),
+        expect.anything(),
+      );
+      expect(result.llmContent).toContain('Found 1 match for pattern "world"');
+      expect(result.llmContent).toContain('File: fileA.txt');
+      expect(result.llmContent).toContain('L2: second line with world');
+      // Note: Ripgrep JSON output for context lines doesn't include line numbers for context lines directly
+      // The current parsing only extracts the matched line, so we only assert on that.
     });
+  });
+
+  describe('getDescription', () => {
+    it.each([
+      {
+        name: 'pattern only',
+        params: { pattern: 'testPattern' },
+        expected: "'testPattern' within ./",
+      },
+      {
+        name: 'pattern and include',
+        params: { pattern: 'testPattern', include: '*.ts' },
+        expected: "'testPattern' in *.ts within ./",
+      },
+      {
+        name: 'root path in description',
+        params: { pattern: 'testPattern', dir_path: '.' },
+        expected: "'testPattern' within ./",
+      },
+    ])(
+      'should generate correct description with $name',
+      ({ params, expected }) => {
+        const invocation = grepTool.build(params);
+        expect(invocation.getDescription()).toBe(expected);
+      },
+    );
 
     it('should generate correct description with pattern and path', async () => {
       const dirPath = path.join(tempRootDir, 'src', 'app');
       await fs.mkdir(dirPath, { recursive: true });
       const params: RipGrepToolParams = {
         pattern: 'testPattern',
-        path: path.join('src', 'app'),
+        dir_path: path.join('src', 'app'),
       };
       const invocation = grepTool.build(params);
-      // The path will be relative to the tempRootDir, so we check for containment.
       expect(invocation.getDescription()).toContain("'testPattern' within");
       expect(invocation.getDescription()).toContain(path.join('src', 'app'));
     });
 
-    it('should indicate searching across all workspace directories when no path specified', () => {
-      // Create a mock config with multiple directories
+    it('should use ./ when no path is specified (defaults to CWD)', () => {
       const multiDirConfig = {
         getTargetDir: () => tempRootDir,
         getWorkspaceContext: () =>
@@ -1394,9 +1742,7 @@ describe('RipGrepTool', () => {
       const multiDirGrepTool = new RipGrepTool(multiDirConfig);
       const params: RipGrepToolParams = { pattern: 'testPattern' };
       const invocation = multiDirGrepTool.build(params);
-      expect(invocation.getDescription()).toBe(
-        "'testPattern' across all workspace directories",
-      );
+      expect(invocation.getDescription()).toBe("'testPattern' within ./");
     });
 
     it('should generate correct description with pattern, include, and path', async () => {
@@ -1405,7 +1751,7 @@ describe('RipGrepTool', () => {
       const params: RipGrepToolParams = {
         pattern: 'testPattern',
         include: '*.ts',
-        path: path.join('src', 'app'),
+        dir_path: path.join('src', 'app'),
       };
       const invocation = grepTool.build(params);
       expect(invocation.getDescription()).toContain(
@@ -1413,12 +1759,6 @@ describe('RipGrepTool', () => {
       );
       expect(invocation.getDescription()).toContain(path.join('src', 'app'));
     });
-
-    it('should use ./ for root path in description', () => {
-      const params: RipGrepToolParams = { pattern: 'testPattern', path: '.' };
-      const invocation = grepTool.build(params);
-      expect(invocation.getDescription()).toBe("'testPattern' within ./");
-    });
   });
 });
 afterAll(() => {
