@@ -15,12 +15,24 @@ import {
   type Mock,
 } from 'vitest';
 
+const mockPlatform = vi.hoisted(() => vi.fn());
+
 const mockShellExecutionService = vi.hoisted(() => vi.fn());
 vi.mock('../services/shellExecutionService.js', () => ({
   ShellExecutionService: { execute: mockShellExecutionService },
 }));
-vi.mock('fs');
-vi.mock('os');
+
+vi.mock('node:os', async (importOriginal) => {
+  const actualOs = await importOriginal<typeof os>();
+  return {
+    ...actualOs,
+    default: {
+      ...actualOs,
+      platform: mockPlatform,
+    },
+    platform: mockPlatform,
+  };
+});
 vi.mock('crypto');
 vi.mock('../utils/summarizer.js');
 
@@ -43,41 +55,49 @@ import * as summarizer from '../utils/summarizer.js';
 import { ToolErrorType } from './tool-error.js';
 import { ToolConfirmationOutcome } from './tools.js';
 import { OUTPUT_UPDATE_INTERVAL_MS } from './shell.js';
-import { createMockWorkspaceContext } from '../test-utils/mockWorkspaceContext.js';
 import { SHELL_TOOL_NAME } from './tool-names.js';
+import { WorkspaceContext } from '../utils/workspaceContext.js';
 
 const originalComSpec = process.env['ComSpec'];
 const itWindowsOnly = process.platform === 'win32' ? it : it.skip;
 
 describe('ShellTool', () => {
+  beforeAll(async () => {
+    await initializeShellParsers();
+  });
+
   let shellTool: ShellTool;
   let mockConfig: Config;
   let mockShellOutputCallback: (event: ShellOutputEvent) => void;
   let resolveExecutionPromise: (result: ShellExecutionResult) => void;
+  let tempRootDir: string;
 
   beforeEach(() => {
     vi.clearAllMocks();
 
+    tempRootDir = fs.mkdtempSync(path.join(os.tmpdir(), 'shell-test-'));
+    fs.mkdirSync(path.join(tempRootDir, 'subdir'));
+
     mockConfig = {
       getAllowedTools: vi.fn().mockReturnValue([]),
       getApprovalMode: vi.fn().mockReturnValue('strict'),
       getCoreTools: vi.fn().mockReturnValue([]),
-      getExcludeTools: vi.fn().mockReturnValue([]),
+      getExcludeTools: vi.fn().mockReturnValue(new Set([])),
       getDebugMode: vi.fn().mockReturnValue(false),
-      getTargetDir: vi.fn().mockReturnValue('/test/dir'),
+      getTargetDir: vi.fn().mockReturnValue(tempRootDir),
       getSummarizeToolOutputConfig: vi.fn().mockReturnValue(undefined),
       getWorkspaceContext: vi
         .fn()
-        .mockReturnValue(createMockWorkspaceContext('/test/dir')),
+        .mockReturnValue(new WorkspaceContext(tempRootDir)),
       getGeminiClient: vi.fn(),
       getEnableInteractiveShell: vi.fn().mockReturnValue(false),
       isInteractive: vi.fn().mockReturnValue(true),
+      getShellToolInactivityTimeout: vi.fn().mockReturnValue(300000),
     } as unknown as Config;
 
     shellTool = new ShellTool(mockConfig);
 
-    vi.mocked(os.platform).mockReturnValue('linux');
-    vi.mocked(os.tmpdir).mockReturnValue('/tmp');
+    mockPlatform.mockReturnValue('linux');
     (vi.mocked(crypto.randomBytes) as Mock).mockReturnValue(
       Buffer.from('abcdef', 'hex'),
     );
@@ -97,6 +117,9 @@ describe('ShellTool', () => {
   });
 
   afterEach(() => {
+    if (fs.existsSync(tempRootDir)) {
+      fs.rmSync(tempRootDir, { recursive: true, force: true });
+    }
     if (originalComSpec === undefined) {
       delete process.env['ComSpec'];
     } else {
@@ -135,30 +158,27 @@ describe('ShellTool', () => {
       );
     });
 
-    it('should throw an error for a relative directory path', () => {
-      expect(() =>
-        shellTool.build({ command: 'ls', directory: 'rel/path' }),
-      ).toThrow('Directory must be an absolute path.');
+    it('should return an invocation for a valid relative directory path', () => {
+      const invocation = shellTool.build({
+        command: 'ls',
+        dir_path: 'subdir',
+      });
+      expect(invocation).toBeDefined();
     });
 
     it('should throw an error for a directory outside the workspace', () => {
-      (mockConfig.getWorkspaceContext as Mock).mockReturnValue(
-        createMockWorkspaceContext('/test/dir', ['/another/workspace']),
-      );
+      const outsidePath = path.resolve(tempRootDir, '../outside');
       expect(() =>
-        shellTool.build({ command: 'ls', directory: '/not/in/workspace' }),
+        shellTool.build({ command: 'ls', dir_path: outsidePath }),
       ).toThrow(
-        "Directory '/not/in/workspace' is not within any of the registered workspace directories.",
+        `Directory '${outsidePath}' is not within any of the registered workspace directories.`,
       );
     });
 
     it('should return an invocation for a valid absolute directory path', () => {
-      (mockConfig.getWorkspaceContext as Mock).mockReturnValue(
-        createMockWorkspaceContext('/test/dir', ['/another/workspace']),
-      );
       const invocation = shellTool.build({
         command: 'ls',
-        directory: '/test/dir/subdir',
+        dir_path: path.join(tempRootDir, 'subdir'),
       });
       expect(invocation).toBeDefined();
     });
@@ -189,32 +209,31 @@ describe('ShellTool', () => {
       const promise = invocation.execute(mockAbortSignal);
       resolveShellExecution({ pid: 54321 });
 
-      vi.mocked(fs.existsSync).mockReturnValue(true);
-      vi.mocked(fs.readFileSync).mockReturnValue(`54321${EOL}54322${EOL}`); // Service PID and background PID
+      // Simulate pgrep output file creation by the shell command
+      const tmpFile = path.join(os.tmpdir(), 'shell_pgrep_abcdef.tmp');
+      fs.writeFileSync(tmpFile, `54321${EOL}54322${EOL}`);
 
       const result = await promise;
 
-      const tmpFile = path.join(os.tmpdir(), 'shell_pgrep_abcdef.tmp');
       const wrappedCommand = `{ my-command & }; __code=$?; pgrep -g 0 >${tmpFile} 2>&1; exit $__code;`;
       expect(mockShellExecutionService).toHaveBeenCalledWith(
         wrappedCommand,
-        '/test/dir',
+        tempRootDir,
         expect.any(Function),
-        mockAbortSignal,
+        expect.any(AbortSignal),
         false,
-        {},
+        { pager: 'cat' },
       );
       expect(result.llmContent).toContain('Background PIDs: 54322');
-      expect(vi.mocked(fs.unlinkSync)).toHaveBeenCalledWith(tmpFile);
+      // The file should be deleted by the tool
+      expect(fs.existsSync(tmpFile)).toBe(false);
     });
 
-    it('should use the provided directory as cwd', async () => {
-      (mockConfig.getWorkspaceContext as Mock).mockReturnValue(
-        createMockWorkspaceContext('/test/dir'),
-      );
+    it('should use the provided absolute directory as cwd', async () => {
+      const subdir = path.join(tempRootDir, 'subdir');
       const invocation = shellTool.build({
         command: 'ls',
-        directory: '/test/dir/subdir',
+        dir_path: subdir,
       });
       const promise = invocation.execute(mockAbortSignal);
       resolveShellExecution();
@@ -224,39 +243,64 @@ describe('ShellTool', () => {
       const wrappedCommand = `{ ls; }; __code=$?; pgrep -g 0 >${tmpFile} 2>&1; exit $__code;`;
       expect(mockShellExecutionService).toHaveBeenCalledWith(
         wrappedCommand,
-        '/test/dir/subdir',
+        subdir,
         expect.any(Function),
-        mockAbortSignal,
+        expect.any(AbortSignal),
         false,
-        {},
+        { pager: 'cat' },
       );
     });
 
-    itWindowsOnly('should not wrap command on windows', async () => {
-      vi.mocked(os.platform).mockReturnValue('win32');
-      const invocation = shellTool.build({ command: 'dir' });
-      const promise = invocation.execute(mockAbortSignal);
-      resolveShellExecution({
-        rawOutput: Buffer.from(''),
-        output: '',
-        exitCode: 0,
-        signal: null,
-        error: null,
-        aborted: false,
-        pid: 12345,
-        executionMethod: 'child_process',
+    it('should use the provided relative directory as cwd', async () => {
+      const invocation = shellTool.build({
+        command: 'ls',
+        dir_path: 'subdir',
       });
+      const promise = invocation.execute(mockAbortSignal);
+      resolveShellExecution();
       await promise;
+
+      const tmpFile = path.join(os.tmpdir(), 'shell_pgrep_abcdef.tmp');
+      const wrappedCommand = `{ ls; }; __code=$?; pgrep -g 0 >${tmpFile} 2>&1; exit $__code;`;
       expect(mockShellExecutionService).toHaveBeenCalledWith(
-        'dir',
-        '/test/dir',
+        wrappedCommand,
+        path.join(tempRootDir, 'subdir'),
         expect.any(Function),
-        mockAbortSignal,
+        expect.any(AbortSignal),
         false,
-        {},
+        { pager: 'cat' },
       );
     });
 
+    itWindowsOnly(
+      'should not wrap command on windows',
+      async () => {
+        mockPlatform.mockReturnValue('win32');
+        const invocation = shellTool.build({ command: 'dir' });
+        const promise = invocation.execute(mockAbortSignal);
+        resolveShellExecution({
+          rawOutput: Buffer.from(''),
+          output: '',
+          exitCode: 0,
+          signal: null,
+          error: null,
+          aborted: false,
+          pid: 12345,
+          executionMethod: 'child_process',
+        });
+        await promise;
+        expect(mockShellExecutionService).toHaveBeenCalledWith(
+          'dir',
+          tempRootDir,
+          expect.any(Function),
+          expect.any(AbortSignal),
+          false,
+          { pager: 'cat' },
+        );
+      },
+      20000,
+    );
+
     it('should format error messages correctly', async () => {
       const error = new Error('wrapped command failed');
       const invocation = shellTool.build({ command: 'user-command' });
@@ -299,12 +343,6 @@ describe('ShellTool', () => {
       );
     });
 
-    it('should throw an error for invalid directory', () => {
-      expect(() =>
-        shellTool.build({ command: 'ls', directory: 'nonexistent' }),
-      ).toThrow('Directory must be an absolute path.');
-    });
-
     it('should summarize output when configured', async () => {
       (mockConfig.getSummarizeToolOutputConfig as Mock).mockReturnValue({
         [SHELL_TOOL_NAME]: { tokenBudget: 1000 },
@@ -329,27 +367,54 @@ describe('ShellTool', () => {
       const result = await promise;
 
       expect(summarizer.summarizeToolOutput).toHaveBeenCalledWith(
+        mockConfig,
+        { model: 'summarizer-shell' },
         expect.any(String),
         mockConfig.getGeminiClient(),
         mockAbortSignal,
-        1000,
       );
       expect(result.llmContent).toBe('summarized output');
       expect(result.returnDisplay).toBe('long output');
     });
 
+    it('should NOT start a timeout if timeoutMs is <= 0', async () => {
+      // Mock the timeout config to be 0
+      (mockConfig.getShellToolInactivityTimeout as Mock).mockReturnValue(0);
+
+      vi.useFakeTimers();
+
+      const invocation = shellTool.build({ command: 'sleep 10' });
+      const promise = invocation.execute(mockAbortSignal);
+
+      // Verify no timeout logic is triggered even after a long time
+      resolveShellExecution({
+        output: 'finished',
+        exitCode: 0,
+      });
+
+      await promise;
+      // If we got here without aborting/timing out logic interfering, we're good.
+      // We can also verify that setTimeout was NOT called for the inactivity timeout.
+      // However, since we don't have direct access to the internal `resetTimeout`,
+      // we can infer success by the fact it didn't abort.
+
+      vi.useRealTimers();
+    });
+
     it('should clean up the temp file on synchronous execution error', async () => {
       const error = new Error('sync spawn error');
       mockShellExecutionService.mockImplementation(() => {
+        // Create the temp file before throwing to simulate it being left behind
+        const tmpFile = path.join(os.tmpdir(), 'shell_pgrep_abcdef.tmp');
+        fs.writeFileSync(tmpFile, '');
         throw error;
       });
-      vi.mocked(fs.existsSync).mockReturnValue(true); // Pretend the file exists
 
       const invocation = shellTool.build({ command: 'a-command' });
       await expect(invocation.execute(mockAbortSignal)).rejects.toThrow(error);
 
       const tmpFile = path.join(os.tmpdir(), 'shell_pgrep_abcdef.tmp');
-      expect(vi.mocked(fs.unlinkSync)).toHaveBeenCalledWith(tmpFile);
+      expect(fs.existsSync(tmpFile)).toBe(false);
     });
 
     describe('Streaming to `updateOutput`', () => {
@@ -480,23 +545,32 @@ describe('ShellTool', () => {
           invocation.shouldConfirmExecute(new AbortController().signal),
         ).rejects.toThrow('wc');
       });
+
+      it('should require all segments of a chained command to be allowlisted', async () => {
+        (mockConfig.getAllowedTools as Mock).mockReturnValue([
+          'ShellTool(echo)',
+        ]);
+        const invocation = shellTool.build({ command: 'echo "foo" && ls -l' });
+        await expect(
+          invocation.shouldConfirmExecute(new AbortController().signal),
+        ).rejects.toThrow(
+          'Command "echo "foo" && ls -l" is not in the list of allowed tools for non-interactive mode.',
+        );
+      });
     });
   });
 
   describe('getDescription', () => {
     it('should return the windows description when on windows', () => {
-      vi.mocked(os.platform).mockReturnValue('win32');
+      mockPlatform.mockReturnValue('win32');
       const shellTool = new ShellTool(mockConfig);
       expect(shellTool.description).toMatchSnapshot();
     });
 
     it('should return the non-windows description when not on windows', () => {
-      vi.mocked(os.platform).mockReturnValue('linux');
+      mockPlatform.mockReturnValue('linux');
       const shellTool = new ShellTool(mockConfig);
       expect(shellTool.description).toMatchSnapshot();
     });
   });
 });
-beforeAll(async () => {
-  await initializeShellParsers();
-});
