@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import type { Content, GenerateContentConfig } from '@google/genai';
+import type { Content } from '@google/genai';
 import type { GeminiClient } from '../core/client.js';
 import type { BaseLlmClient } from '../core/baseLlmClient.js';
 import type { EditToolParams } from '../tools/edit.js';
@@ -16,7 +16,6 @@ import {
   WRITE_FILE_TOOL_NAME,
 } from '../tools/tool-names.js';
 import { LruCache } from './LruCache.js';
-import { DEFAULT_GEMINI_FLASH_LITE_MODEL } from '../config/models.js';
 import {
   isFunctionResponse,
   isFunctionCall,
@@ -24,13 +23,6 @@ import {
 import * as fs from 'node:fs';
 import { promptIdContext } from './promptIdContext.js';
 
-const EDIT_MODEL = DEFAULT_GEMINI_FLASH_LITE_MODEL;
-const EDIT_CONFIG: GenerateContentConfig = {
-  thinkingConfig: {
-    thinkingBudget: 0,
-  },
-};
-
 const CODE_CORRECTION_SYSTEM_PROMPT = `
 You are an expert code-editing assistant. Your task is to analyze a failed edit attempt and provide a corrected version of the text snippets.
 The correction should be as minimal as possible, staying very close to the original.
@@ -420,11 +412,10 @@ Return ONLY the corrected target snippet in the specified JSON format with the k
 
   try {
     const result = await baseLlmClient.generateJson({
+      modelConfigKey: { model: 'edit-corrector' },
       contents,
       schema: OLD_STRING_CORRECTION_SCHEMA,
       abortSignal,
-      model: EDIT_MODEL,
-      config: EDIT_CONFIG,
       systemInstruction: CODE_CORRECTION_SYSTEM_PROMPT,
       promptId: getPromptId(),
     });
@@ -510,11 +501,10 @@ Return ONLY the corrected string in the specified JSON format with the key 'corr
 
   try {
     const result = await baseLlmClient.generateJson({
+      modelConfigKey: { model: 'edit-corrector' },
       contents,
       schema: NEW_STRING_CORRECTION_SCHEMA,
       abortSignal,
-      model: EDIT_MODEL,
-      config: EDIT_CONFIG,
       systemInstruction: CODE_CORRECTION_SYSTEM_PROMPT,
       promptId: getPromptId(),
     });
@@ -581,11 +571,10 @@ Return ONLY the corrected string in the specified JSON format with the key 'corr
 
   try {
     const result = await baseLlmClient.generateJson({
+      modelConfigKey: { model: 'edit-corrector' },
       contents,
       schema: CORRECT_NEW_STRING_ESCAPING_SCHEMA,
       abortSignal,
-      model: EDIT_MODEL,
-      config: EDIT_CONFIG,
       systemInstruction: CODE_CORRECTION_SYSTEM_PROMPT,
       promptId: getPromptId(),
     });
@@ -649,11 +638,10 @@ Return ONLY the corrected string in the specified JSON format with the key 'corr
 
   try {
     const result = await baseLlmClient.generateJson({
+      modelConfigKey: { model: 'edit-corrector' },
       contents,
       schema: CORRECT_STRING_ESCAPING_SCHEMA,
       abortSignal,
-      model: EDIT_MODEL,
-      config: EDIT_CONFIG,
       systemInstruction: CODE_CORRECTION_SYSTEM_PROMPT,
       promptId: getPromptId(),
     });
