@@ -15,8 +15,10 @@ import { processImports } from './memoryImportProcessor.js';
 import type { FileFilteringOptions } from '../config/constants.js';
 import { DEFAULT_MEMORY_FILE_FILTERING_OPTIONS } from '../config/constants.js';
 import { GEMINI_DIR } from './paths.js';
-import type { GeminiCLIExtension } from '../config/config.js';
+import type { ExtensionLoader } from './extensionLoader.js';
 import { debugLogger } from './debugLogger.js';
+import type { Config } from '../config/config.js';
+import { CoreEvent, coreEvents } from './events.js';
 
 // Simple console logger, similar to the one previously in CLI's config.ts
 // TODO: Integrate with a more robust server-side logger if available/appropriate.
@@ -32,7 +34,7 @@ const logger = {
     console.error('[ERROR] [MemoryDiscovery]', ...args),
 };
 
-interface GeminiFileContent {
+export interface GeminiFileContent {
   filePath: string;
   content: string | null;
 }
@@ -302,7 +304,7 @@ async function readGeminiMdFiles(
   return results;
 }
 
-function concatenateInstructions(
+export function concatenateInstructions(
   instructionContents: GeminiFileContent[],
   // CWD is needed to resolve relative paths for display markers
   currentWorkingDirectoryForDisplay: string,
@@ -323,6 +325,143 @@ function concatenateInstructions(
     .join('\n\n');
 }
 
+export interface MemoryLoadResult {
+  files: Array<{ path: string; content: string }>;
+}
+
+export async function loadGlobalMemory(
+  debugMode: boolean = false,
+): Promise<MemoryLoadResult> {
+  const userHome = homedir();
+  const geminiMdFilenames = getAllGeminiMdFilenames();
+
+  const accessChecks = geminiMdFilenames.map(async (filename) => {
+    const globalPath = path.join(userHome, GEMINI_DIR, filename);
+    try {
+      await fs.access(globalPath, fsSync.constants.R_OK);
+      if (debugMode) {
+        logger.debug(`Found global memory file: ${globalPath}`);
+      }
+      return globalPath;
+    } catch {
+      debugLogger.debug('A global memory file was not found.');
+      return null;
+    }
+  });
+
+  const foundPaths = (await Promise.all(accessChecks)).filter(
+    (p): p is string => p !== null,
+  );
+
+  const contents = await readGeminiMdFiles(foundPaths, debugMode, 'tree');
+
+  return {
+    files: contents
+      .filter((item) => item.content !== null)
+      .map((item) => ({
+        path: item.filePath,
+        content: item.content as string,
+      })),
+  };
+}
+
+/**
+ * Traverses upward from startDir to stopDir, finding all GEMINI.md variants.
+ *
+ * Files are ordered by directory level (root to leaf), with all filename
+ * variants grouped together per directory.
+ */
+async function findUpwardGeminiFiles(
+  startDir: string,
+  stopDir: string,
+  debugMode: boolean,
+): Promise<string[]> {
+  const upwardPaths: string[] = [];
+  let currentDir = path.resolve(startDir);
+  const resolvedStopDir = path.resolve(stopDir);
+  const geminiMdFilenames = getAllGeminiMdFilenames();
+  const globalGeminiDir = path.join(homedir(), GEMINI_DIR);
+
+  if (debugMode) {
+    logger.debug(
+      `Starting upward search from ${currentDir} stopping at ${resolvedStopDir}`,
+    );
+  }
+
+  while (true) {
+    if (currentDir === globalGeminiDir) {
+      break;
+    }
+
+    // Parallelize checks for all filename variants in the current directory
+    const accessChecks = geminiMdFilenames.map(async (filename) => {
+      const potentialPath = path.join(currentDir, filename);
+      try {
+        await fs.access(potentialPath, fsSync.constants.R_OK);
+        return potentialPath;
+      } catch {
+        return null;
+      }
+    });
+
+    const foundPathsInDir = (await Promise.all(accessChecks)).filter(
+      (p): p is string => p !== null,
+    );
+
+    upwardPaths.unshift(...foundPathsInDir);
+
+    if (
+      currentDir === resolvedStopDir ||
+      currentDir === path.dirname(currentDir)
+    ) {
+      break;
+    }
+    currentDir = path.dirname(currentDir);
+  }
+  return upwardPaths;
+}
+
+export async function loadEnvironmentMemory(
+  trustedRoots: string[],
+  extensionLoader: ExtensionLoader,
+  debugMode: boolean = false,
+): Promise<MemoryLoadResult> {
+  const allPaths = new Set<string>();
+
+  // Trusted Roots Upward Traversal (Parallelized)
+  const traversalPromises = trustedRoots.map(async (root) => {
+    const resolvedRoot = path.resolve(root);
+    if (debugMode) {
+      logger.debug(
+        `Loading environment memory for trusted root: ${resolvedRoot} (Stopping exactly here)`,
+      );
+    }
+    return findUpwardGeminiFiles(resolvedRoot, resolvedRoot, debugMode);
+  });
+
+  const pathArrays = await Promise.all(traversalPromises);
+  pathArrays.flat().forEach((p) => allPaths.add(p));
+
+  // Extensions
+  const extensionPaths = extensionLoader
+    .getExtensions()
+    .filter((ext) => ext.isActive)
+    .flatMap((ext) => ext.contextFiles);
+  extensionPaths.forEach((p) => allPaths.add(p));
+
+  const sortedPaths = Array.from(allPaths).sort();
+  const contents = await readGeminiMdFiles(sortedPaths, debugMode, 'tree');
+
+  return {
+    files: contents
+      .filter((item) => item.content !== null)
+      .map((item) => ({
+        path: item.filePath,
+        content: item.content as string,
+      })),
+  };
+}
+
 export interface LoadServerHierarchicalMemoryResponse {
   memoryContent: string;
   fileCount: number;
@@ -338,12 +477,21 @@ export async function loadServerHierarchicalMemory(
   includeDirectoriesToReadGemini: readonly string[],
   debugMode: boolean,
   fileService: FileDiscoveryService,
-  extensions: GeminiCLIExtension[],
+  extensionLoader: ExtensionLoader,
   folderTrust: boolean,
   importFormat: 'flat' | 'tree' = 'tree',
   fileFilteringOptions?: FileFilteringOptions,
   maxDirs: number = 200,
 ): Promise<LoadServerHierarchicalMemoryResponse> {
+  // FIX: Use real, canonical paths for a reliable comparison to handle symlinks.
+  const realCwd = await fs.realpath(path.resolve(currentWorkingDirectory));
+  const realHome = await fs.realpath(path.resolve(homedir()));
+  const isHomeDirectory = realCwd === realHome;
+
+  // If it is the home directory, pass an empty string to the core memory
+  // function to signal that it should skip the workspace search.
+  currentWorkingDirectory = isHomeDirectory ? '' : currentWorkingDirectory;
+
   if (debugMode)
     logger.debug(
       `Loading server hierarchical memory for CWD: ${currentWorkingDirectory} (importFormat: ${importFormat})`,
@@ -365,7 +513,8 @@ export async function loadServerHierarchicalMemory(
 
   // Add extension file paths separately since they may be conditionally enabled.
   filePaths.push(
-    ...extensions
+    ...extensionLoader
+      .getExtensions()
       .filter((ext) => ext.isActive)
       .flatMap((ext) => ext.contextFiles),
   );
@@ -399,3 +548,100 @@ export async function loadServerHierarchicalMemory(
     filePaths,
   };
 }
+
+/**
+ * Loads the hierarchical memory and resets the state of `config` as needed such
+ * that it reflects the new memory.
+ *
+ * Returns the result of the call to `loadHierarchicalGeminiMemory`.
+ */
+export async function refreshServerHierarchicalMemory(config: Config) {
+  const result = await loadServerHierarchicalMemory(
+    config.getWorkingDir(),
+    config.shouldLoadMemoryFromIncludeDirectories()
+      ? config.getWorkspaceContext().getDirectories()
+      : [],
+    config.getDebugMode(),
+    config.getFileService(),
+    config.getExtensionLoader(),
+    config.isTrustedFolder(),
+    config.getImportFormat(),
+    config.getFileFilteringOptions(),
+    config.getDiscoveryMaxDirs(),
+  );
+  const mcpInstructions =
+    config.getMcpClientManager()?.getMcpInstructions() || '';
+  const finalMemory = [result.memoryContent, mcpInstructions.trimStart()]
+    .filter(Boolean)
+    .join('\n\n');
+  config.setUserMemory(finalMemory);
+  config.setGeminiMdFileCount(result.fileCount);
+  config.setGeminiMdFilePaths(result.filePaths);
+  coreEvents.emit(CoreEvent.MemoryChanged, result);
+  return result;
+}
+
+export async function loadJitSubdirectoryMemory(
+  targetPath: string,
+  trustedRoots: string[],
+  alreadyLoadedPaths: Set<string>,
+  debugMode: boolean = false,
+): Promise<MemoryLoadResult> {
+  const resolvedTarget = path.resolve(targetPath);
+  let bestRoot: string | null = null;
+
+  // Find the deepest trusted root that contains the target path
+  for (const root of trustedRoots) {
+    const resolvedRoot = path.resolve(root);
+    if (
+      resolvedTarget.startsWith(resolvedRoot) &&
+      (!bestRoot || resolvedRoot.length > bestRoot.length)
+    ) {
+      bestRoot = resolvedRoot;
+    }
+  }
+
+  if (!bestRoot) {
+    if (debugMode) {
+      logger.debug(
+        `JIT memory skipped: ${resolvedTarget} is not in any trusted root.`,
+      );
+    }
+    return { files: [] };
+  }
+
+  if (debugMode) {
+    logger.debug(
+      `Loading JIT memory for ${resolvedTarget} (Trusted root: ${bestRoot})`,
+    );
+  }
+
+  // Traverse from target up to the trusted root
+  const potentialPaths = await findUpwardGeminiFiles(
+    resolvedTarget,
+    bestRoot,
+    debugMode,
+  );
+
+  // Filter out already loaded paths
+  const newPaths = potentialPaths.filter((p) => !alreadyLoadedPaths.has(p));
+
+  if (newPaths.length === 0) {
+    return { files: [] };
+  }
+
+  if (debugMode) {
+    logger.debug(`Found new JIT memory files: ${JSON.stringify(newPaths)}`);
+  }
+
+  const contents = await readGeminiMdFiles(newPaths, debugMode, 'tree');
+
+  return {
+    files: contents
+      .filter((item) => item.content !== null)
+      .map((item) => ({
+        path: item.filePath,
+        content: item.content as string,
+      })),
+  };
+}
